@@ -6,21 +6,26 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.09 00:00:00                  #
+# Updated Date: 2025.02.01 02:00:00                  #
 # ================================================== #
 
 __author__ = "Marcin Szczygliński"
-__copyright__ = "Copyright 2024, Marcin Szczygliński"
+__copyright__ = "Copyright 2025, Marcin Szczygliński"
 __credits__ = ["Marcin Szczygliński"]
 __license__ = "MIT"
-__version__ = "2.4.5"
-__build__ = "2024.11.09"
+__version__ = "2.5.1"
+__build__ = "2025.02.01"
 __maintainer__ = "Marcin Szczygliński"
 __github__ = "https://github.com/szczyglis-dev/py-gpt"
+__report__ = "https://github.com/szczyglis-dev/py-gpt/issues"
 __website__ = "https://pygpt.net"
 __pypi__ = "https://pypi.org/project/pygpt-net"
 __snap__ = "https://snapcraft.io/pygpt"
 __donate__ = "https://pygpt.net/#donate"
 __documentation__ = "https://pygpt.readthedocs.io"
 __discord__ = "https://pygpt.net/discord"
+__ms_store__ = "https://apps.microsoft.com/detail/xp99r4mx3x65vq"
 __email__ = "info@pygpt.net"
+__donate_coffee__ = "https://pygpt.net/donate/buymeacoffee"
+__donate_paypal__ = "https://pygpt.net/donate/paypal"
+__donate_github__ = "https://github.com/sponsors/szczyglis-dev"
