@@ -6,9 +6,14 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.08 23:00:00                  #
+# Updated Date: 2025.01.31 19:00:00                  #
 # ================================================== #
 
+import os
+
+# disable warnings
+os.environ["TRANSFORMERS_NO_ADVISORY_WARNINGS"] = "1"
+
 from pygpt_net.launcher import Launcher
 
 # plugins
@@ -22,6 +27,7 @@ from pygpt_net.plugin.cmd_files import Plugin as CmdFilesPlugin
 from pygpt_net.plugin.cmd_history import Plugin as CtxHistoryPlugin
 from pygpt_net.plugin.cmd_mouse_control import Plugin as CmdMousePlugin
 from pygpt_net.plugin.cmd_serial import Plugin as CmdSerialPlugin
+from pygpt_net.plugin.cmd_system import Plugin as CmdSystemPlugin
 from pygpt_net.plugin.cmd_web import Plugin as CmdWebPlugin
 from pygpt_net.plugin.crontab import Plugin as CrontabPlugin
 from pygpt_net.plugin.extra_prompt import Plugin as ExtraPromptPlugin
@@ -31,12 +37,22 @@ from pygpt_net.plugin.openai_dalle import Plugin as OpenAIDallePlugin
 from pygpt_net.plugin.openai_vision import Plugin as OpenAIVisionPlugin
 from pygpt_net.plugin.real_time import Plugin as RealTimePlugin
 from pygpt_net.plugin.agent import Plugin as AgentPlugin
+from pygpt_net.plugin.mailer import Plugin as MailerPlugin
+
+# agents (Llama-index)
+from pygpt_net.provider.agents.openai import OpenAIAgent
+from pygpt_net.provider.agents.openai_assistant import OpenAIAssistantAgent
+from pygpt_net.provider.agents.planner import PlannerAgent
+from pygpt_net.provider.agents.react import ReactAgent
 
 # LLM wrapper providers (langchain, llama-index, embeddings)
 from pygpt_net.provider.llms.anthropic import AnthropicLLM
 from pygpt_net.provider.llms.azure_openai import AzureOpenAILLM
-from pygpt_net.provider.llms.hugging_face import HuggingFaceLLM
+from pygpt_net.provider.llms.deepseek_api import DeepseekApiLLM
 from pygpt_net.provider.llms.google import GoogleLLM
+from pygpt_net.provider.llms.hugging_face import HuggingFaceLLM
+from pygpt_net.provider.llms.hugging_face_api import HuggingFaceApiLLM
+from pygpt_net.provider.llms.local import LocalLLM
 from pygpt_net.provider.llms.ollama import OllamaLLM
 from pygpt_net.provider.llms.openai import OpenAILLM
 
@@ -100,7 +116,7 @@ from pygpt_net.tools.code_interpreter import CodeInterpreter as CodeInterpreterT
 from pygpt_net.tools.image_viewer import ImageViewer as ImageViewerTool
 from pygpt_net.tools.media_player import MediaPlayer as MediaPlayerTool
 from pygpt_net.tools.text_editor import TextEditor as TextEditorTool
-
+from pygpt_net.tools.html_canvas import HtmlCanvas as HtmlCanvasTool
 
 def run(**kwargs):
     """
@@ -152,6 +168,10 @@ def run(**kwargs):
 
     - Pass a list with the web provider instances as the 'web' keyword argument.
 
+    To register a custom agent:
+
+    - Pass a list with the agent instances as the 'agents' keyword argument.
+
     To register a custom tool:
 
     - Pass a list with the tool instances as the 'tools' keyword argument.
@@ -194,6 +214,9 @@ def run(**kwargs):
         web = [
             CustomWebSearch(),
         ]
+        agents = [
+            CustomAgent(),
+        ]
         tools = [
             CustomTool(),
         ]
@@ -206,6 +229,7 @@ def run(**kwargs):
             audio_input=audio_input,
             audio_output=audio_output,
             web=web,
+            agents=agents,
             tools=tools,
         )
 
@@ -294,6 +318,7 @@ def run(**kwargs):
     launcher.add_plugin(CmdWebPlugin())
     launcher.add_plugin(CmdFilesPlugin())
     launcher.add_plugin(CmdCodeInterpreterPlugin())
+    launcher.add_plugin(CmdSystemPlugin())
     launcher.add_plugin(CmdCustomCommandPlugin())
     launcher.add_plugin(CmdApiPlugin())
     launcher.add_plugin(CmdSerialPlugin())
@@ -302,6 +327,7 @@ def run(**kwargs):
     launcher.add_plugin(OpenAIDallePlugin())
     launcher.add_plugin(OpenAIVisionPlugin())
     launcher.add_plugin(IdxLlamaIndexPlugin())
+    launcher.add_plugin(MailerPlugin())
     launcher.add_plugin(CrontabPlugin())
 
     # register custom plugins
@@ -314,9 +340,12 @@ def run(**kwargs):
     launcher.add_llm(OpenAILLM())
     launcher.add_llm(AzureOpenAILLM())
     launcher.add_llm(AnthropicLLM())
-    launcher.add_llm(HuggingFaceLLM())
     launcher.add_llm(GoogleLLM())
+    launcher.add_llm(HuggingFaceLLM())
+    launcher.add_llm(HuggingFaceApiLLM())
+    launcher.add_llm(LocalLLM())
     launcher.add_llm(OllamaLLM())
+    launcher.add_llm(DeepseekApiLLM())
 
     # register custom langchain and llama-index LLMs
     llms = kwargs.get('llms', None)
@@ -337,6 +366,18 @@ def run(**kwargs):
         for store in vector_stores:
             launcher.add_vector_store(store)
 
+    # register base llama-index agents
+    launcher.add_agent(OpenAIAgent())
+    launcher.add_agent(OpenAIAssistantAgent())
+    launcher.add_agent(PlannerAgent())
+    launcher.add_agent(ReactAgent())
+
+    # register custom agents
+    agents = kwargs.get('agents', None)
+    if isinstance(agents, list):
+        for agent in agents:
+            launcher.add_agent(agent)
+
     # register base tools
     launcher.add_tool(IndexerTool())
     launcher.add_tool(MediaPlayerTool())
@@ -344,6 +385,7 @@ def run(**kwargs):
     launcher.add_tool(TextEditorTool())
     launcher.add_tool(AudioTranscriberTool())
     launcher.add_tool(CodeInterpreterTool())
+    launcher.add_tool(HtmlCanvasTool())
 
     # register custom tools
     tools = kwargs.get('tools', None)
