@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.07 23:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
 import os
@@ -14,8 +14,7 @@ import os
 from PySide6.QtGui import QTextCursor
 from PySide6.QtWidgets import QFileDialog, QApplication
 
-from pygpt_net.core.access.events import AppEvent
-from pygpt_net.core.dispatcher import Event
+from pygpt_net.core.events import Event, AppEvent, RenderEvent
 from pygpt_net.item.ctx import CtxItem
 from pygpt_net.utils import trans
 
@@ -69,7 +68,11 @@ class Common:
         is_timestamp = self.window.core.config.get('output_timestamp')
         self.window.ui.nodes['output.timestamp'].setChecked(is_timestamp)
         if is_timestamp:
-            self.window.controller.chat.render.on_enable_timestamp(self.initialized)
+            data = {
+                "initialized": self.initialized,
+            }
+            event = RenderEvent(RenderEvent.ON_TS_ENABLE, data)
+            self.window.dispatch(event)
 
         # raw (plain) output
         plain = self.window.core.config.get('render.plain')
@@ -83,15 +86,22 @@ class Common:
                 self.window.ui.nodes['output'][pid].setVisible(True)
                 self.window.ui.nodes['output_plain'][pid].setVisible(False)
 
-        self.window.controller.chat.render.switch(self.initialized)  # switch renderer if needed
+        event = RenderEvent(RenderEvent.ON_SWITCH)
+        self.window.dispatch(event)  # switch renderer if needed
 
         # edit icons
+        """
         if self.window.core.config.has('ctx.edit_icons'):
             self.window.ui.nodes['output.edit'].setChecked(self.window.core.config.get('ctx.edit_icons'))
+            data = {
+                "initialized": self.initialized,
+            }
             if self.window.core.config.get('ctx.edit_icons'):
-                self.window.controller.chat.render.on_enable_edit(self.initialized)
+                event = RenderEvent(RenderEvent.ON_EDIT_ENABLE, data)
             else:
-                self.window.controller.chat.render.on_disable_edit(self.initialized)
+                event = RenderEvent(RenderEvent.ON_EDIT_DISABLE, data)
+            self.window.dispatch(event)
+        """
 
         # images generation
         if self.window.core.config.get('img_raw'):
@@ -103,7 +113,11 @@ class Common:
         self.window.ui.nodes['input'].setFocus()
         self.initialized = True
 
-    def append_to_input(self, text: str, separator: str = "\n"):
+    def append_to_input(
+            self,
+            text: str,
+            separator: str = "\n"
+    ):
         """
         Append text to input
 
@@ -137,7 +151,7 @@ class Common:
         """
         Toggle stream
 
-        :param value: value of the checkbox
+        :param value: True if enabled
         """
         self.window.core.config.set('stream', value)
 
@@ -145,7 +159,7 @@ class Common:
         """
         Toggle cmd enabled
 
-        :param value: value of the checkbox
+        :param value: True if enabled
         """
         self.window.core.config.set('cmd', value)
 
@@ -155,13 +169,13 @@ class Common:
         else:
             self.window.controller.command.stop = False
 
-        self.window.controller.ui.update_tokens()  # update tokens counters
+        self.window.controller.ui.update()  # update tokens counters
 
     def toggle_send_clear(self, value: bool):
         """
         Toggle send clear
 
-        :param value: value of the checkbox
+        :param value: True if enabled
         """
         self.window.core.config.set('send_clear', value)
 
@@ -169,7 +183,7 @@ class Common:
         """
         Toggle send with shift
 
-        :param value: value of the checkbox
+        :param value: True if enabled
         """
         self.window.core.config.set('send_mode', value)
 
@@ -186,6 +200,7 @@ class Common:
     def unlock_input(self):
         """Unlock input"""
         self.window.controller.chat.input.locked = False
+        self.window.controller.chat.input.generating = False  # unlock
         self.window.ui.nodes['input.send_btn'].setEnabled(True)
         self.window.ui.nodes['input.stop_btn'].setVisible(False)
 
@@ -200,17 +215,18 @@ class Common:
         mode = self.window.core.config.get('mode')
         if mode == "assistant":
             finish = False
-        if (self.window.controller.agent.enabled() and
-                (not self.window.controller.agent.flow.finished or self.window.controller.agent.flow.stop)):
+        if (self.window.controller.agent.legacy.enabled() and
+                (not self.window.controller.agent.legacy.finished or self.window.controller.agent.legacy.stop)):
             unlock = False
         if (self.window.controller.agent.experts.enabled() and
                 self.window.core.experts.has_calls(ctx)):
             unlock = False
-        if self.window.controller.chat.reply.waiting():
+        if self.window.controller.kernel.stack.waiting():
+            unlock = False
+        if ctx.has_commands():
             unlock = False
         return unlock
 
-
     def stop(self, exit: bool = False):
         """
         Stop all
@@ -222,23 +238,26 @@ class Common:
         event = Event(Event.FORCE_STOP, {
             "value": True,
         })
-        self.window.core.dispatcher.dispatch(event)  # stop event
+        self.window.dispatch(event)  # stop event
         event = Event(Event.AUDIO_INPUT_TOGGLE, {
             "value": False,
         })
-        self.window.controller.chat.reply.clear()  # pause reply stack
+        self.window.controller.kernel.stack.clear()  # pause reply stack
         self.window.controller.agent.experts.stop()
-        self.window.controller.agent.flow.on_stop()
+        self.window.controller.agent.legacy.on_stop()
         self.window.controller.assistant.threads.stop = True
         self.window.controller.assistant.threads.reset()  # reset run and func calls
-        self.window.core.dispatcher.dispatch(event)  # stop audio input
-        self.window.controller.chat.input.stop = True
-        self.window.controller.chat.render.tool_output_end()  # show waiting
+        self.window.dispatch(event)  # stop audio input
+        self.window.controller.kernel.halt = True
+
+        event = RenderEvent(RenderEvent.TOOL_END)
+        self.window.dispatch(event)  # show waiting
+
         self.window.core.gpt.stop()
         self.unlock_input()
 
         self.window.controller.chat.input.generating = False
-        self.window.ui.status(trans('status.stopped'))
+        self.window.update_status(trans('status.stopped'))
         self.window.stateChanged.emit(self.window.STATE_IDLE)
 
         # remotely stop assistant
@@ -249,15 +268,7 @@ class Common:
                 self.window.core.debug.log(e)
 
         if not exit:
-            self.window.core.dispatcher.dispatch(AppEvent(AppEvent.INPUT_STOPPED))  # app event
-
-    def stopped(self) -> bool:
-        """
-        Check if input is stopped
-
-        :return: True if input is stopped
-        """
-        return self.window.controller.chat.input.stop
+            self.window.dispatch(AppEvent(AppEvent.INPUT_STOPPED))  # app event
 
     def check_api_key(self) -> bool:
         """
@@ -268,7 +279,7 @@ class Common:
         result = True
         if self.window.core.config.get('api_key') is None or self.window.core.config.get('api_key') == '':
             self.window.controller.launcher.show_api_monit()
-            self.window.ui.status("Missing API KEY!")
+            self.window.update_status("Missing API KEY!")
             result = False
         return result
 
@@ -280,10 +291,14 @@ class Common:
         """
         self.window.core.config.set('output_timestamp', value)
         self.window.core.config.save()
+        data = {
+            "initialized": True
+        }
         if value:
-            self.window.controller.chat.render.on_enable_timestamp(live=True)
+            event = RenderEvent(RenderEvent.ON_TS_ENABLE, data)
         else:
-            self.window.controller.chat.render.on_disable_timestamp(live=True)
+            event = RenderEvent(RenderEvent.ON_TS_DISABLE, data)
+        self.window.dispatch(event)
 
     def toggle_raw(self, value: bool):
         """
@@ -302,7 +317,8 @@ class Common:
                 'value': value
             },
         )
-        self.window.controller.chat.render.switch()
+        event = RenderEvent(RenderEvent.ON_SWITCH)
+        self.window.dispatch(event)
 
         # restore previous font size
         self.window.controller.ui.update_font_size()
@@ -315,10 +331,14 @@ class Common:
         """
         self.window.core.config.set('ctx.edit_icons', value)
         self.window.core.config.save()
+        data = {
+            "initialized": True,
+        }
         if value:
-            self.window.controller.chat.render.on_enable_edit(live=True)
+            event = RenderEvent(RenderEvent.ON_EDIT_ENABLE, data)
         else:
-            self.window.controller.chat.render.on_disable_edit(live=True)
+            event = RenderEvent(RenderEvent.ON_EDIT_DISABLE, data)
+        self.window.dispatch(event)
 
     def img_enable_raw(self):
         """Enable help for images"""
@@ -341,7 +361,11 @@ class Common:
         else:
             self.img_enable_raw()
 
-    def save_text(self, text: str, type: str = "txt"):
+    def save_text(
+            self,
+            text: str,
+            type: str = "txt"
+    ):
         """
         Save text to file
 
@@ -369,7 +393,7 @@ class Common:
             )
             with open(file_name, 'w', encoding="utf-8") as f:
                 f.write(str(text).strip())
-            self.window.ui.status(trans('status.saved') + ": " + os.path.basename(file_name))
+            self.window.update_status(trans('status.saved') + ": " + os.path.basename(file_name))
 
     def show_response_tokens(self, ctx: CtxItem):
         """
@@ -380,7 +404,7 @@ class Common:
         extra_data = ""
         if ctx.is_vision:
             extra_data = " (VISION)"
-        self.window.ui.status(
+        self.window.update_status(
             trans('status.tokens') + ": {} + {} = {}{}".
             format(
                 ctx.input_tokens,
