@@ -6,14 +6,17 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.08 23:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
-from PySide6.QtWidgets import  QApplication
+from typing import Optional, List
 
-from pygpt_net.core.bridge import BridgeContext
+from PySide6.QtCore import Slot
+
+from pygpt_net.core.bridge.context import BridgeContext
+from pygpt_net.core.types import MODE_IMAGE
 from pygpt_net.item.ctx import CtxItem
-from pygpt_net.core.dispatcher import Event
+from pygpt_net.core.events import Event, KernelEvent, RenderEvent
 from pygpt_net.utils import trans
 
 
@@ -29,8 +32,8 @@ class Image:
     def send(
             self,
             text: str,
-            prev_ctx: CtxItem = None,
-            parent_id: int = None,
+            prev_ctx: Optional[CtxItem] = None,
+            parent_id: Optional[int] = None,
     ) -> CtxItem:
         """
         Send prompt for image generate
@@ -52,10 +55,12 @@ class Image:
         if model_data.id == 'dall-e-3':
             num = 1
 
-        self.window.ui.status(trans('status.sending'))
+        self.window.update_status(trans('status.sending'))
 
         # create ctx item
         ctx = CtxItem()
+        ctx.current = True  # mark as current context item
+        ctx.meta = self.window.core.ctx.get_current_meta()  # CtxMeta (owner object)
         ctx.set_input(text, self.window.core.config.get('user_name'))
         ctx.prev_ctx = prev_ctx  # store previous context item
         ctx.live = True
@@ -63,14 +68,27 @@ class Image:
         # event: context before
         event = Event(Event.CTX_BEFORE)
         event.ctx = ctx
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
 
         # add ctx to DB
         self.window.core.ctx.add(ctx)
-        self.window.controller.chat.render.append_input(ctx.meta, ctx)
 
-        # process events to update UI
-        QApplication.processEvents()
+        # render: begin
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+            "stream": False,
+        }
+        event = RenderEvent(RenderEvent.STREAM_BEGIN, data)
+        self.window.dispatch(event)
+
+        # append input to chat
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+        }
+        event = RenderEvent(RenderEvent.INPUT_APPEND, data)
+        self.window.dispatch(event)
 
         # handle ctx name (generate title from summary if not initialized)
         if self.window.core.config.get('ctx.auto_summary'):
@@ -79,7 +97,7 @@ class Image:
         # generate image
         bridge_context = BridgeContext(
             ctx=ctx,
-            mode="image",
+            mode=MODE_IMAGE,
             model=model_data,  # model instance
             prompt=text,
         )
@@ -87,18 +105,25 @@ class Image:
             "num": num,
         }
         try:
-            self.window.core.bridge.call(
-                context=bridge_context,
-                extra=extra,
-            )
+            event = KernelEvent(KernelEvent.REQUEST, {
+                'context': bridge_context,
+                'extra': extra,
+            })
+            self.window.dispatch(event)
         except Exception as e:
             self.window.core.debug.log(e)
             self.window.ui.dialogs.alert(e)
-            self.window.ui.status(trans('status.error'))
+            self.window.update_status(trans('status.error'))
 
         return ctx
 
-    def handle_response(self, ctx: CtxItem, paths: list, prompt: str):
+    @Slot(object, list, str)
+    def handle_response(
+            self,
+            ctx: CtxItem,
+            paths: List[str],
+            prompt: str
+    ):
         """
         Handle response
 
@@ -106,6 +131,9 @@ class Image:
         :param paths: list with paths to downloaded images
         :param prompt: prompt used to generate images
         """
+        self.window.dispatch(KernelEvent(KernelEvent.STATE_IDLE, {
+            "id": "img",
+        }))
         string = ""
         i = 1
         for path in paths:
@@ -126,25 +154,26 @@ class Image:
         # event: after context
         event = Event(Event.CTX_AFTER)
         event.ctx = ctx
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
 
         # store last mode (in text mode this is handled in send_text)
         mode = self.window.core.config.get('mode')
         self.window.core.ctx.post_update(mode)  # post update context, store last mode, etc.
-        self.window.controller.chat.render.append_output(ctx.meta, ctx)
         self.window.core.ctx.store()  # save current ctx to DB
-        self.window.ui.status(trans('status.img.generated'))
+        self.window.update_status(trans('status.img.generated'))
 
         # update ctx in DB
         self.window.core.ctx.update_item(ctx)
 
-        # append extra output to chat
-        self.window.controller.chat.render.append_extra(ctx.meta, ctx)
-        self.window.controller.chat.render.end_extra(ctx.meta, ctx)
+        self.window.controller.chat.common.unlock_input()  # unlock input
 
-        self.window.stateChanged.emit(self.window.STATE_IDLE)  # set state to idle
-
-    def handle_response_inline(self, ctx: CtxItem, paths: list, prompt: str):
+    @Slot(object, list, str)
+    def handle_response_inline(
+            self,
+            ctx: CtxItem,
+            paths: List[str],
+            prompt: str
+    ):
         """
         Handle inline response
 
@@ -152,6 +181,9 @@ class Image:
         :param paths: list with paths to downloaded images
         :param prompt: prompt used to generate images
         """
+        self.window.dispatch(KernelEvent(KernelEvent.STATE_IDLE, {
+            "id": "img",
+        }))
         string = ""
         i = 1
         for path in paths:
@@ -162,7 +194,7 @@ class Image:
         ctx.images = local_urls  # save images paths in ctx item here
         
         self.window.core.ctx.update_item(ctx)  # update in DB
-        self.window.ui.status(trans('status.img.generated'))  # update status
+        self.window.update_status(trans('status.img.generated'))  # update status
 
         # WARNING:
         # if internal (sync) mode, then re-send OK status response, if not, append only img result
@@ -178,14 +210,40 @@ class Image:
                     "paths": paths,
                 }
             )
-            ctx.reply = True
-            self.window.controller.chat.render.append_extra(ctx.meta, ctx)  # show image first
-            self.window.controller.chat.render.end_extra(ctx.meta, ctx)
-            self.window.core.dispatcher.reply(ctx, flush=True)
+            ctx.reply = False
+            data = {
+                "meta": ctx.meta,
+                "ctx": ctx,
+            }
+            event = RenderEvent(RenderEvent.EXTRA_APPEND, data)
+            self.window.dispatch(event)  # show image first
+
+            event = RenderEvent(RenderEvent.EXTRA_END, data)
+            self.window.dispatch(event)  # end extra
+
+            context = BridgeContext()
+            context.ctx = ctx
+            extra = {
+                "flush": True,
+            }
+            event = KernelEvent(KernelEvent.REPLY_ADD, {
+                'context': context,
+                'extra': extra,
+            })
+            self.window.dispatch(event)
+            self.window.controller.chat.common.unlock_input()  # unlock input
             return
 
         # NOT internal-mode, user called, so append only img output to chat (show images now)
-        self.window.controller.chat.render.append_extra(ctx.meta, ctx)
-        self.window.controller.chat.render.end_extra(ctx.meta, ctx)
 
-        self.window.stateChanged.emit(self.window.STATE_IDLE)  # set state to idle
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+        }
+        event = RenderEvent(RenderEvent.EXTRA_APPEND, data)
+        self.window.dispatch(event)  # show image first
+
+        event = RenderEvent(RenderEvent.EXTRA_END, data)
+        self.window.dispatch(event)  # end extra
+
+        self.window.controller.chat.common.unlock_input()  # unlock input
