@@ -6,11 +6,17 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.05 23:00:00                  #
+# Updated Date: 2024.12.14 00:00:00                  #
 # ================================================== #
 
-from pygpt_net.core.access.events import AppEvent
-from pygpt_net.core.dispatcher import Event
+from typing import Any
+
+from pygpt_net.core.types import (
+    MODE_AGENT,
+    MODE_ASSISTANT,
+    MODE_IMAGE,
+)
+from pygpt_net.core.events import Event, AppEvent, RenderEvent, KernelEvent
 from pygpt_net.item.ctx import CtxItem
 
 
@@ -22,9 +28,17 @@ class Output:
         :param window: Window instance
         """
         self.window = window
-        self.not_stream_modes = ['assistant', 'img']
+        self.not_stream_modes = [
+            MODE_ASSISTANT,
+            MODE_IMAGE,
+        ]
 
-    def handle(self, ctx: CtxItem, mode: str, stream_mode: bool = False):
+    def handle(
+            self,
+            ctx: CtxItem,
+            mode: str,
+            stream_mode: bool = False
+    ):
         """
         Handle response from LLM
 
@@ -54,22 +68,42 @@ class Output:
                 self.log("Ignoring tool call because command received...")
 
         # agent mode
-        if mode == 'agent':
-            self.window.controller.agent.flow.on_ctx_after(ctx)
+        if mode == MODE_AGENT:
+            self.window.controller.agent.legacy.on_ctx_after(ctx)
 
         # event: context after
         event = Event(Event.CTX_AFTER)
         event.ctx = ctx
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
 
         self.log("Appending output to chat window...")
 
         # only append output if not in stream mode, TODO: plugin output add
         if not stream_mode:
             if stream_enabled:  # use global stream settings here to persist previously added input
-                self.window.controller.chat.render.append_input(ctx.meta, ctx, flush=True, append=True)
-            self.window.controller.chat.render.append_output(ctx.meta, ctx)
-            self.window.controller.chat.render.append_extra(ctx.meta, ctx, True)  # + icons
+                data = {
+                    "meta": ctx.meta,
+                    "ctx": ctx,
+                    "flush": True,
+                    "append": True,
+                }
+                event = RenderEvent(RenderEvent.INPUT_APPEND, data)
+                self.window.dispatch(event)
+
+            data = {
+                "meta": ctx.meta,
+                "ctx": ctx,
+            }
+            event = RenderEvent(RenderEvent.OUTPUT_APPEND, data)
+            self.window.dispatch(event)
+
+            data = {
+                "meta": ctx.meta,
+                "ctx": ctx,
+                "footer": True,
+            }
+            event = RenderEvent(RenderEvent.EXTRA_APPEND, data)
+            self.window.dispatch(event)  # + icons
 
         self.handle_complete(ctx)
 
@@ -87,12 +121,21 @@ class Output:
         # update response tokens
         self.window.controller.chat.common.show_response_tokens(ctx)
 
+        # handle audio output
+        self.window.controller.chat.audio.handle_output(ctx)
+
         # store to history
         if self.window.core.config.get('store_history'):
             self.window.core.history.append(ctx, "output")
 
         if self.window.controller.chat.common.can_unlock(ctx):
-            self.window.controller.chat.common.unlock_input()  # unlock input
+            if not self.window.controller.kernel.stopped():
+                self.window.controller.chat.common.unlock_input()  # unlock input
+
+        # reset state to: idle
+        self.window.dispatch(KernelEvent(KernelEvent.STATE_IDLE, {
+            "id": "chat",
+        }))
 
     def post_handle(
             self,
@@ -100,7 +143,8 @@ class Output:
             mode: str,
             stream: bool = False,
             reply: bool = False,
-            internal: bool = False):
+            internal: bool = False
+    ):
         """
         Post handle results
 
@@ -111,7 +155,7 @@ class Output:
         :param internal: is internal
         """
         # if commands enabled: post-execute commands (not assistant mode)
-        if mode != "assistant":
+        if mode != MODE_ASSISTANT:
             ctx.clear_reply()  # reset results
 
             # extract expert mentions and handle experts reply
@@ -125,12 +169,19 @@ class Output:
 
         # render: end
         if ctx.sub_calls == 0:  # if no experts called
-            self.window.controller.chat.render.end(ctx.meta, ctx, stream=stream)
+            data = {
+                "meta": ctx.meta,
+                "ctx": ctx,
+                "stream": stream,
+            }
+            event = RenderEvent(RenderEvent.END, data)
+            self.window.dispatch(event)
 
         # don't unlock input and leave stop btn if assistant mode or if agent/autonomous is enabled
         # send btn will be unlocked in agent mode on stop
         if self.window.controller.chat.common.can_unlock(ctx):
-            self.window.controller.chat.common.unlock_input()  # unlock input
+            if not self.window.controller.kernel.stopped():
+                self.window.controller.chat.common.unlock_input()  # unlock input
 
         # handle ctx name (generate title from summary if not initialized)
         if not reply and not internal:  # don't call if reply or internal mode
@@ -138,18 +189,22 @@ class Output:
                 self.log("Calling for prepare context name...")
                 self.window.controller.ctx.prepare_name(ctx)  # async
 
-    def handle_end(self, ctx: CtxItem, mode: str, has_attachments: bool = False):
+    def handle_end(
+            self,
+            ctx: CtxItem,
+            mode: str
+    ):
         """
         Handle context end (finish output)
 
         :param ctx: CtxItem
         :param mode: mode
-        :param has_attachments: has attachments
         """
         # clear attachments after send, only if attachments has been provided before send
-        if has_attachments:
-            if self.window.core.config.get('attachments_send_clear'):
-                self.window.controller.attachment.clear(True, auto=True)
+        auto_clear = self.window.core.config.get('attachments_send_clear')
+        if self.window.controller.attachment.clear_allowed(ctx):
+            if auto_clear and not self.window.controller.attachment.is_locked():
+                self.window.controller.attachment.clear(force=True, auto=True)
                 self.window.controller.attachment.update()
                 self.log("Attachments cleared.")  # log
 
@@ -159,10 +214,10 @@ class Output:
             self.log("Context: END.")
 
         # agent mode
-        if mode == 'agent':
+        if mode == MODE_AGENT:
             agent_iterations = int(self.window.core.config.get("agent.iterations"))
             self.log("Agent: ctx end, iterations: {}".format(agent_iterations))
-            self.window.controller.agent.flow.on_ctx_end(
+            self.window.controller.agent.legacy.on_ctx_end(
                 ctx,
                 iterations=agent_iterations,
             )
@@ -170,29 +225,31 @@ class Output:
         # event: context end
         event = Event(Event.CTX_END)
         event.ctx = ctx
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
         self.window.controller.ui.update_tokens()  # update UI
         self.window.controller.chat.input.generating = False  # unlock
 
         if (mode not in self.window.controller.chat.input.no_ctx_idx_modes
-                and not self.window.controller.agent.enabled()):
+                and not self.window.controller.agent.legacy.enabled()):
             self.window.controller.idx.on_ctx_end(ctx, mode=mode)  # update ctx DB index
             # disabled in agent mode here to prevent loops, handled in agent flow internally if agent mode
 
         self.log("End.")
-        self.window.core.dispatcher.dispatch(AppEvent(AppEvent.CTX_END))  # app event
+        self.window.dispatch(AppEvent(AppEvent.CTX_END))  # app event
 
         # restore state to idle if no errors
         if self.window.state != self.window.STATE_ERROR:
             self.window.stateChanged.emit(self.window.STATE_IDLE)
 
-        if mode != "assistant":
-            self.window.controller.chat.reply.handle()  # handle reply
+        if mode != MODE_ASSISTANT:
+            self.window.controller.kernel.stack.handle()  # handle reply
+            event = RenderEvent(RenderEvent.RELOAD)
+            self.window.dispatch(event)  # reload chat window
 
-    def log(self, data: any):
+    def log(self, data: Any):
         """
         Log data to debug
 
         :param data: Data to log
         """
-        self.window.core.debug.info(data)
+        self.window.controller.chat.log(data)
