@@ -6,11 +6,14 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.05 23:00:00                  #
+# Updated Date: 2025.01.31 22:00:00                  #
 # ================================================== #
 
+from typing import Any
+
 from PySide6.QtWidgets import QApplication
 
+from pygpt_net.core.events import RenderEvent
 from pygpt_net.item.ctx import CtxItem
 
 
@@ -32,22 +35,31 @@ class Stream:
         output = ""
         output_tokens = 0
         begin = True
+        error = None
 
         # chunks: stream begin
-        self.window.controller.chat.render.stream_begin(ctx.meta, ctx)
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+        }
+        event = RenderEvent(RenderEvent.STREAM_BEGIN, data)
+        self.window.dispatch(event)
 
         # read stream
         try:
             if ctx.stream is not None:
-                self.log("Reading stream...")  # log
+                self.log("[chat] Stream begin...")  # log
 
                 tool_calls = []
 
                 for chunk in ctx.stream:
                     # if force stop then break
-                    if self.window.controller.chat.input.stop:
+                    if self.window.controller.kernel.stopped():
                         break
 
+                    if error is not None:
+                        break  # break if error
+
                     response = None
                     chunk_type = "raw"
                     if (hasattr(chunk, 'choices')
@@ -63,12 +75,15 @@ class Stream:
                     elif (hasattr(chunk, 'content')
                           and chunk.content is not None):
                         chunk_type = "langchain_chat"
+                    elif (hasattr(chunk, 'delta')
+                          and chunk.delta is not None):
+                        chunk_type = "llama_chat"
 
                     # OpenAI chat completion
                     if chunk_type == "api_chat":
                         if chunk.choices[0].delta and chunk.choices[0].delta.content is not None:
                             response = chunk.choices[0].delta.content
-                        elif chunk.choices[0].delta and chunk.choices[0].delta.tool_calls:
+                        if chunk.choices[0].delta and chunk.choices[0].delta.tool_calls:
                             tool_chunks = chunk.choices[0].delta.tool_calls
                             for tool_chunk in tool_chunks:
                                 if len(tool_calls) <= tool_chunk.index:
@@ -98,24 +113,47 @@ class Stream:
                     # langchain chat
                     elif chunk_type == "langchain_chat":
                         if chunk.content is not None:
-                            response = chunk.content
+                            response = str(chunk.content)
+
+                    # llama chat
+                    elif chunk_type == "llama_chat":
+                        if chunk.delta is not None:
+                            response = str(chunk.delta)
+                        tool_chunks = chunk.message.additional_kwargs.get("tool_calls", [])
+                        if tool_chunks:
+                            for tool_chunk in tool_chunks:
+                                args = tool_chunk.function.arguments
+                                if not args:
+                                    args = "{}"  # JSON encoded
+                                tool_call = {
+                                        "id": tool_chunk.id,
+                                        "type": "function",
+                                        "function": {
+                                            "name": tool_chunk.function.name,
+                                            "arguments": args
+                                        }
+                                    }
+                                tool_calls.clear()
+                                tool_calls.append(tool_call)
 
                     # raw text: llama-index and langchain completion
                     else:
                         if chunk is not None:
-                            response = chunk
+                            response = str(chunk)
 
-                    if response is not None:
+                    if response is not None and response != "":
                         if begin and response == "":  # prevent empty beginning
                             continue
                         output += response
                         output_tokens += 1
-                        self.window.controller.chat.render.append_chunk(
-                            ctx.meta,
-                            ctx,
-                            response,
-                            begin,
-                        )
+                        data = {
+                            "meta": ctx.meta,
+                            "ctx": ctx,
+                            "chunk": response,
+                            "begin": begin,
+                        }
+                        event = RenderEvent(RenderEvent.STREAM_APPEND, data)
+                        self.window.dispatch(event)
                         QApplication.processEvents()  # process events to update UI after each chunk
                         begin = False
 
@@ -125,6 +163,7 @@ class Stream:
 
         except Exception as e:
             self.window.core.debug.log(e)
+            error = e
 
         self.window.controller.ui.update_tokens()  # update UI tokens
 
@@ -133,12 +172,20 @@ class Stream:
         ctx.set_tokens(ctx.input_tokens, output_tokens)
 
         # chunks: stream end
-        self.window.controller.chat.render.stream_end(ctx.meta, ctx)
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+        }
+        event = RenderEvent(RenderEvent.STREAM_END, data)
+        self.window.dispatch(event)
 
         # log
-        self.log("End of stream.")
+        self.log("[chat] Stream end.")
 
-    def log(self, data: any):
+        if error is not None:
+            raise error  # raise error if any, to display in UI
+
+    def log(self, data: Any):
         """
         Log data to debug
 
