@@ -6,9 +6,19 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.04.11 22:00:00                  #
+# Updated Date: 2024.11.21 20:00:00                  #
 # ================================================== #
 
+from pygpt_net.core.types import (
+    MODE_AGENT,
+    MODE_AGENT_LLAMA,
+    MODE_CHAT,
+    MODE_COMPLETION,
+    MODE_LANGCHAIN,
+    MODE_LLAMA_INDEX,
+    MODE_VISION,
+)
+
 class Vision:
     def __init__(self, window=None):
         """
@@ -20,31 +30,18 @@ class Vision:
         self.is_enabled = False
         self.is_available = False
         self.allowed_modes = [
-            "chat",
-            "completion",
-            "langchain",
-            "llama_index",
-            "agent",
+            MODE_CHAT,
+            MODE_COMPLETION,
+            MODE_LANGCHAIN,
+            MODE_LLAMA_INDEX,
+            MODE_AGENT,
+            MODE_AGENT_LLAMA,
         ]
 
     def setup(self):
         """Set up UI"""
         pass
 
-    def toggle(self, value: bool, clear: bool = True):
-        """
-        Toggle inline vision
-
-        :param value: value of the checkbox
-        :param clear: clear attachments
-        """
-        if not value:
-            self.disable()  # disable vision
-        else:
-            self.enable()  # enable vision keep
-
-        self.window.controller.ui.update_tokens()  # update tokens
-
     def show_inline(self):
         """Show inline vision checkbox"""
         self.window.ui.nodes['inline.vision'].setVisible(True)  # show vision checkbox
@@ -53,24 +50,6 @@ class Vision:
         """Hide inline vision checkbox"""
         self.window.ui.nodes['inline.vision'].setVisible(False)  # hide vision checkbox
 
-    def enable(self):
-        """Enable inline vision"""
-        self.is_enabled = True
-        self.window.ui.nodes['inline.vision'].setChecked(True)
-
-    def enabled(self) -> bool:
-        """
-        Check if inline vision is enabled
-
-        :return: True if enabled
-        """
-        return self.is_enabled
-
-    def disable(self):
-        """Disable inline vision"""
-        self.is_enabled = False
-        self.window.ui.nodes['inline.vision'].setChecked(False)
-
     def available(self):
         """Set vision content available"""
         self.is_available = True
@@ -84,15 +63,17 @@ class Vision:
         mode = self.window.core.config.get('mode')
         model = self.window.core.config.get('model')
         model_data = self.window.core.models.get(model)
-        if mode == "chat" and "vision" in model_data.mode:
+        if mode in [MODE_AGENT, MODE_AGENT_LLAMA]:
+            return  # disallow change in agent modes
+        if mode == MODE_CHAT and MODE_VISION in model_data.mode:
             return  # abort if vision is already allowed
-        if mode == 'vision':
+        if mode == MODE_VISION:
             return
         # abort if vision is already enabled
         if not self.window.controller.plugins.is_enabled('openai_vision') \
                 or (self.window.controller.plugins.is_enabled('openai_vision')
                     and mode not in self.allowed_modes):
-            self.window.controller.mode.set('vision')
+            self.window.controller.mode.set(MODE_VISION)
 
     def allowed(self) -> bool:
         """
@@ -105,10 +86,3 @@ class Vision:
             return True
         return False
 
-    def update(self):
-        """Update vision content on mode change"""
-        mode = self.window.core.config.get('mode')
-        if mode in self.allowed_modes:
-            self.show_inline()
-        else:
-            self.hide_inline()
