@@ -6,11 +6,16 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.05 23:00:00                  #
+# Updated Date: 2024.12.16 01:00:00                  #
 # ================================================== #
 
-from pygments.styles import get_all_styles
+from typing import Dict, Any, List
 
+from pygpt_net.core.types import (
+    MODE_AGENT_LLAMA,
+    MODE_LANGCHAIN,
+    MODE_LLAMA_INDEX,
+)
 from pygpt_net.utils import trans
 
 
@@ -23,7 +28,7 @@ class Placeholder:
         """
         self.window = window
 
-    def apply(self, option: dict):
+    def apply(self, option: Dict[str, Any]):
         """
         Apply placeholders to option
 
@@ -49,7 +54,7 @@ class Placeholder:
             if "use" in option and option["use"] is not None:
                 option["keys"] = self.apply_by_id(option["use"])
 
-    def apply_by_id(self, id: str) -> list:
+    def apply_by_id(self, id: str) -> List[Dict[str, str]]:
         """
         Apply placeholders by id
 
@@ -79,8 +84,12 @@ class Placeholder:
             return self.get_var_types()
         elif id == "agent_modes":
             return self.get_agent_modes()
+        elif id == "agent_provider":
+            return self.get_agent_providers()
         elif id == "syntax_styles":
             return self.get_syntax_styles()
+        elif id == "styles":
+            return self.get_styles()
         elif id == "idx":
             return self.get_idx()
         elif id == "keys":
@@ -93,36 +102,74 @@ class Placeholder:
             return self.get_speech_synthesis_actions()
         elif id == "voice_control_actions":
             return self.get_voice_control_actions()
+        elif id == "audio_input_devices":
+            return self.get_audio_input_devices()
+        elif id == "audio_tts_whisper_voices":
+            return self.get_audio_tts_whisper_voices()
         else:
             return []
 
-    def get_langchain_providers(self) -> list:
+    def get_audio_tts_whisper_voices(self) -> List[Dict[str, str]]:
         """
-        Get langchain placeholders list
+        Get audio TTS whisper voices list
 
         :return: placeholders list
         """
-        ids = self.window.core.llm.get_ids("langchain")
+        voices = self.window.core.audio.whisper.get_voices()
+        data = []
+        for voice in voices:
+            data.append({voice: voice})
+        return data
+
+    def get_audio_input_devices(self) -> List[Dict[str, str]]:
+        """
+        Get audio input devices list
+
+        :return: placeholders list
+        """
+        devices = self.window.core.audio.get_input_devices()
+        data = []
+        for device in devices:
+            data.append({str(device[0]): device[1]})
+        return data
+
+    def get_langchain_providers(self) -> List[Dict[str, str]]:
+        """
+        Get Langchain LLM provider placeholders list
+
+        :return: placeholders list
+        """
+        ids = self.window.core.llm.get_ids(MODE_LANGCHAIN)
         data = []
-        data.append({'_': '---'})
         for id in ids:
             data.append({id: id})
         return data
 
-    def get_llama_index_providers(self) -> list:
+    def get_llama_index_providers(self) -> List[Dict[str, str]]:
         """
-        Get llama placeholders list
+        Get Llama-index LLM provider placeholders list
 
         :return: placeholders list
         """
-        ids = self.window.core.llm.get_ids("llama_index")
+        ids = self.window.core.llm.get_ids(MODE_LLAMA_INDEX)
         data = []
-        data.append({'_': '---'})
         for id in ids:
             data.append({id: id})
         return data
 
-    def get_embeddings_providers(self) -> list:
+    def get_agent_providers(self) -> List[Dict[str, str]]:
+        """
+        Get Llama-index agent provider placeholders list
+
+        :return: placeholders list
+        """
+        ids = self.window.core.agents.provider.get_providers()
+        data = []
+        for id in ids:
+            data.append({id: id})
+        return data
+
+    def get_embeddings_providers(self) -> List[Dict[str, str]]:
         """
         Get embeddings placeholders list
 
@@ -134,7 +181,7 @@ class Placeholder:
             data.append({id: id})
         return data
 
-    def get_llama_index_chat_modes(self) -> list:
+    def get_llama_index_chat_modes(self) -> List[Dict[str, str]]:
         """
         Get llama chat modes list
 
@@ -150,7 +197,7 @@ class Placeholder:
             {"openai": "openai"},
         ]
 
-    def get_llama_index_loaders(self, type: str = "all") -> list:
+    def get_llama_index_loaders(self, type: str = "all") -> List[Dict[str, str]]:
         """
         Get data loaders placeholders list
 
@@ -172,7 +219,7 @@ class Placeholder:
                     data.append(choice)
         return data
 
-    def get_vector_storage(self) -> list:
+    def get_vector_storage(self) -> List[Dict[str, str]]:
         """
         Get vector storage placeholders list
 
@@ -184,7 +231,7 @@ class Placeholder:
             data.append({id: id})
         return data
 
-    def get_var_types(self) -> list:
+    def get_var_types(self) -> List[Dict[str, str]]:
         """
         Get langchain placeholders list
 
@@ -196,7 +243,7 @@ class Placeholder:
             data.append({type: type})
         return data
 
-    def get_presets(self) -> list:
+    def get_presets(self) -> List[Dict[str, str]]:
         """
         Get presets placeholders list
 
@@ -211,7 +258,7 @@ class Placeholder:
             data.append({id: id})  # TODO: name
         return data
 
-    def get_models(self) -> list:
+    def get_models(self) -> List[Dict[str, str]]:
         """
         Get models placeholders list
 
@@ -223,19 +270,20 @@ class Placeholder:
             data.append({id: id})  # TODO: name
         return data
 
-    def get_agent_modes(self) -> list:
+    def get_agent_modes(self) -> List[Dict[str, str]]:
         """
         Get agent/expert modes placeholders list
 
         :return: Models placeholders list
         """
-        modes = self.window.core.agents.get_allowed_modes()
+        modes = self.window.core.agents.legacy.get_allowed_modes()
         data = []
         for id in modes:
-            data.append({id: id})  # TODO: name
+            name = trans(f"mode.{id}")
+            data.append({id: name})
         return data
 
-    def get_idx(self) -> list:
+    def get_idx(self) -> List[Dict[str, str]]:
         """
         Get indexes placeholders list
 
@@ -244,11 +292,12 @@ class Placeholder:
         indexes = self.window.core.idx.get_idx_ids()
         data = []
         data.append({'_': '---'})
-        for id in indexes:
-            data.append({id: id})
+        for item in indexes:
+            for k, v in item.items():
+                data.append({k: v})
         return data
 
-    def get_syntax_styles(self) -> list:
+    def get_syntax_styles(self) -> List[Dict[str, str]]:
         """
         Get highlighter styles list
 
@@ -261,7 +310,20 @@ class Placeholder:
             data.append({id: id})
         return data
 
-    def get_keys(self) -> list:
+    def get_styles(self) -> List[Dict[str, str]]:
+        """
+        Get styles list
+
+        :return: placeholders list
+        """
+        styles = self.window.controller.theme.common.get_styles_list()
+        styles.sort()
+        data = []
+        for id in styles:
+            data.append({id: id})
+        return data
+
+    def get_keys(self) -> List[Dict[str, str]]:
         """
         Get keys
 
@@ -269,7 +331,7 @@ class Placeholder:
         """
         return self.window.core.access.shortcuts.get_keys_choices()
 
-    def get_modifiers(self) -> list:
+    def get_modifiers(self) -> List[Dict[str, str]]:
         """
         Get modifiers
 
@@ -277,7 +339,7 @@ class Placeholder:
         """
         return self.window.core.access.shortcuts.get_modifiers_choices()
 
-    def get_access_actions(self) -> list:
+    def get_access_actions(self) -> List[Dict[str, str]]:
         """
         Get access actions list
 
@@ -292,7 +354,7 @@ class Placeholder:
         translated_choices.sort(key=lambda x: list(x.values())[0])
         return translated_choices
 
-    def get_speech_synthesis_actions(self) -> list:
+    def get_speech_synthesis_actions(self) -> List[Dict[str, str]]:
         """
         Get speech actions list
 
@@ -307,7 +369,7 @@ class Placeholder:
         translated_choices.sort(key=lambda x: list(x.values())[0])
         return translated_choices
 
-    def get_voice_control_actions(self) -> list:
+    def get_voice_control_actions(self) -> List[Dict[str, str]]:
         """
         Get voice control actions list
 
