@@ -6,13 +6,15 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.07 23:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
-from PySide6.QtCore import QModelIndex
+from typing import Optional, List
 
-from pygpt_net.core.dispatcher import Event
-from pygpt_net.core.access.events import AppEvent
+from PySide6.QtCore import QModelIndex
+from PySide6.QtGui import QStandardItem
+
+from pygpt_net.core.events import Event, AppEvent, RenderEvent
 from pygpt_net.item.ctx import CtxItem
 
 from .common import Common
@@ -20,7 +22,7 @@ from .summarizer import Summarizer
 from .extra import Extra
 
 from pygpt_net.utils import trans
-from pygpt_net.core.tabs import Tab
+from pygpt_net.core.tabs.tab import Tab
 
 
 class Ctx:
@@ -61,20 +63,6 @@ class Ctx:
                 # if no ctx then get first ctx
                 self.window.core.ctx.set_current(self.window.core.ctx.get_first())
 
-        # restore previous ctx
-        # 1) try to load from tabs data - if not exists, load from current
-        tab = self.window.core.tabs.get_first_by_type(Tab.TAB_CHAT)
-        loaded = False
-        if tab is not None:
-            meta_id = tab.data_id
-            if meta_id is not None:
-                self.load(meta_id)
-                loaded = True
-
-        # 2) load from current if not loaded yet
-        if not loaded:
-            self.load(self.window.core.ctx.get_current())
-
         # restore search string if exists
         if self.window.core.config.has("ctx.search.string"):
             string = self.window.core.config.get("ctx.search.string")
@@ -90,7 +78,12 @@ class Ctx:
         self.window.ui.nodes['ctx.list'].collapseAll()  # collapse all items at start
         self.restore_expanded_groups()  # restore expanded groups
 
-    def update(self, reload: bool = True, all: bool = True, select: bool = True):
+    def update(
+            self,
+            reload: bool = True,
+            all: bool = True,
+            select: bool = True
+    ):
         """
         Update ctx list
 
@@ -120,7 +113,14 @@ class Ctx:
         # update calendar ctx list
         self.window.controller.calendar.update(all=False)
 
-    def select(self, id: int, force: bool = False):
+        # update additional context attachments
+        self.window.controller.chat.attachment.update()
+
+    def select(
+            self,
+            id: int,
+            force: bool = False
+    ):
         """
         Select ctx by id
 
@@ -131,13 +131,29 @@ class Ctx:
         self.window.core.ctx.set_current(id)
         if prev_id != id or force:
             self.load(id)
-            self.window.core.dispatcher.dispatch(AppEvent(AppEvent.CTX_SELECTED))  # app event
+            self.window.dispatch(AppEvent(AppEvent.CTX_SELECTED))  # app event
         else:
             # only update current group if defined
             meta = self.window.core.ctx.get_meta_by_id(id)
             if meta is not None:
                 self.set_group(meta.group_id)
+
         self.common.focus_chat()
+        # update additional context attachments
+        self.window.controller.chat.attachment.update()
+
+    def select_on_list_only(self, id: int):
+        """
+        Select ctx by id only on list
+
+        :param id: context meta id
+        """
+        self.window.core.ctx.select(id, restore_model=True)
+        self.window.core.ctx.set_current(id)
+        self.update_list(True)
+        self.select_by_current()
+        self.reload_config(all=False)
+        self.update()
 
     def select_by_idx(self, idx: int):
         """
@@ -161,7 +177,7 @@ class Ctx:
         event = Event(Event.CTX_SELECT, {
             'value': id,
         })
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
 
     def select_by_current(self, focus: bool = False):
         """
@@ -184,7 +200,7 @@ class Ctx:
         self.set_group(None)
         self.window.ui.nodes['ctx.list'].clearSelection()
 
-    def set_group(self, group_id: int = None):
+    def set_group(self, group_id: Optional[int] = None):
         """
         Set current selected group
 
@@ -202,7 +218,11 @@ class Ctx:
         self.group_id = None
         self.new()
 
-    def new(self, force: bool = False, group_id: int = None):
+    def new(
+            self,
+            force: bool = False,
+            group_id: Optional[int] = None
+    ):
         """
         Create new ctx
 
@@ -230,7 +250,11 @@ class Ctx:
         self.update()
 
         # render reset
-        self.window.controller.chat.render.clear(meta)
+        data = {
+            "meta": meta,
+        }
+        event = RenderEvent(RenderEvent.CLEAR, data)
+        self.window.dispatch(event)
 
         if not force:  # only if real click on new context button
             self.window.controller.chat.common.unlock_input()
@@ -251,7 +275,10 @@ class Ctx:
             self.window.controller.ui.tabs.update_title_current(meta.name)
 
         # app event
-        self.window.core.dispatcher.dispatch(AppEvent(AppEvent.CTX_CREATED))
+        self.window.dispatch(AppEvent(AppEvent.CTX_CREATED))
+
+        # switch to new context if non-chat tab
+        self.select(meta.id)
         return meta
 
     def add(self, ctx: CtxItem):
@@ -306,13 +333,19 @@ class Ctx:
     def refresh_output(self):
         """Refresh output"""
         # append ctx to output
-        self.window.controller.chat.render.append_context(
-            self.window.core.ctx.get_current_meta(),
-            self.window.core.ctx.get_items(),
-            clear=True,
-        )
+        data = {
+            "meta": self.window.core.ctx.get_current_meta(),
+            "items": self.window.core.ctx.get_items(),
+            "clear": True,
+        }
+        event = RenderEvent(RenderEvent.CTX_APPEND, data)
+        self.window.dispatch(event)
 
-    def load(self, id: int, restore_model: bool = True):
+    def load(
+            self,
+            id: int,
+            restore_model: bool = True
+    ):
         """
         Load ctx data
 
@@ -328,8 +361,27 @@ class Ctx:
 
         # reset appended data / prepare new ctx
         if meta is not None:
-            self.window.controller.chat.render.on_load(meta)
+            data = {
+                "meta": meta,
+            }
+            event = RenderEvent(RenderEvent.ON_LOAD, data)
+            self.window.dispatch(event)
 
+        self.reload_config()
+
+        # reload ctx list and select current ctx on list, without reloading all
+        self.update(reload=False, all=True)
+
+        # update tab title
+        if meta is not None:
+            self.window.controller.ui.tabs.on_load_ctx(meta)
+
+    def reload_config(self, all: bool = True):
+        """
+        Reload config
+
+        :param all: reload all
+        """
         # get current settings stored in ctx
         thread = self.window.core.ctx.get_thread()
         mode = self.window.core.ctx.get_mode()
@@ -340,8 +392,9 @@ class Ctx:
         # restore thread from ctx
         self.window.core.config.set('assistant_thread', thread)
 
-        # clear before output and append ctx to output
-        self.refresh_output()
+        if all:
+            # clear before output and append ctx to output
+            self.refresh_output()
 
         # switch mode to ctx mode
         if mode is not None:
@@ -366,16 +419,9 @@ class Ctx:
             if model is not None and self.window.core.models.has_model(mode, model):
                 self.window.controller.model.set(mode, model)
 
-        # reload ctx list and select current ctx on list, without reloading all
-        self.update(reload=False, all=True)
-
         # update current ctx label in UI
         self.common.update_label(mode, assistant_id)
 
-        # update tab title
-        if meta is not None:
-            self.window.controller.ui.tabs.update_title_current(meta.name)
-
     def update_ctx(self):
         """Update current ctx mode if allowed"""
         mode = self.window.core.config.get('mode')
@@ -397,7 +443,11 @@ class Ctx:
         # update ctx label
         self.common.update_label(mode, id)
 
-    def delete(self, id: int, force: int = False):
+    def delete(
+            self,
+            id: int,
+            force: bool = False
+    ):
         """
         Delete ctx by idx
 
@@ -422,12 +472,14 @@ class Ctx:
         # delete ctx items from db
         items = self.window.core.ctx.all()
         self.window.core.history.remove_items(items)  # remove txt history items
+        self.window.core.attachments.context.delete_by_meta_id(id)
         self.window.core.ctx.remove(id)  # remove ctx from db
 
         # reset current if current ctx deleted
         if self.window.core.ctx.get_current() == id:
             self.window.core.ctx.clear_current()
-            self.window.controller.chat.render.clear_output()
+            event = RenderEvent(RenderEvent.CLEAR_OUTPUT)
+            self.window.dispatch(event)
         self.update()
 
         # update tab title
@@ -449,7 +501,11 @@ class Ctx:
                 for idx in list(meta.indexes[store_id]):
                     self.window.core.ctx.idx.remove_meta_from_indexed(store_id, id, idx)
 
-    def delete_item(self, id: int, force: bool = False):
+    def delete_item(
+            self,
+            id: int,
+            force: bool = False
+    ):
         """
         Delete ctx item by id
 
@@ -493,6 +549,7 @@ class Ctx:
         self.unselect()
         self.window.core.ctx.truncate()
         self.window.core.history.truncate()
+        self.window.core.attachments.context.truncate()
         self.update()
         self.new()
 
@@ -523,6 +580,7 @@ class Ctx:
         self.window.core.ctx.truncate()
         self.window.core.history.truncate()
         self.window.core.ctx.truncate_groups()
+        self.window.core.attachments.context.truncate()
         self.update()
         self.new()
 
@@ -545,7 +603,11 @@ class Ctx:
         self.window.ui.dialog['rename'].show()
         self.update()
 
-    def set_important(self, id: int, value: bool = True):
+    def set_important(
+            self,
+            id: int,
+            value: bool = True
+    ):
         """
         Set as important
 
@@ -571,7 +633,11 @@ class Ctx:
             return meta.important
         return False
 
-    def set_label(self, id: int, label_id: int):
+    def set_label(
+            self,
+            id: int,
+            label_id: int
+    ):
         """
         Set color label for ctx by idx
 
@@ -672,7 +738,7 @@ class Ctx:
         self.window.ui.nodes['ctx.search'].setText(text)
         self.search_string_change(text)  # make search
 
-    def label_filters_changed(self, labels: list):
+    def label_filters_changed(self, labels: List[int]):
         """
         Filters labels change
 
@@ -704,7 +770,7 @@ class Ctx:
         """
         return self.window.controller.chat.input.generating
 
-    def select_index_by_id(self, id):
+    def select_index_by_id(self, id: int):
         """
         Select item by ID on context list
 
@@ -715,7 +781,11 @@ class Ctx:
         self.window.ui.nodes['ctx.list'].setCurrentIndex(index)
         self.window.ui.nodes['ctx.list'].unlocked = False  # tmp allow change if locked (disable)
 
-    def find_index_by_id(self, item, id):
+    def find_index_by_id(
+            self,
+            item: QStandardItem,
+            id: int
+    ) -> QModelIndex:
         """
         Return index of item with given ID, searching recursively through the model.
 
@@ -731,7 +801,11 @@ class Ctx:
                 return found_index
         return QModelIndex()
 
-    def find_parent_index_by_id(self, item, id):
+    def find_parent_index_by_id(
+            self,
+            item: QStandardItem,
+            id: int
+    ) -> QModelIndex:
         """
         Return index of item with given ID, searching recursively through the model.
 
@@ -747,7 +821,7 @@ class Ctx:
                 return found_index
         return QModelIndex()
 
-    def get_parent_index_by_id(self, id):
+    def get_parent_index_by_id(self, id: int) -> QModelIndex:
         """
         Return QModelIndex of parent item based on its ID.
 
@@ -758,7 +832,11 @@ class Ctx:
         root = model.invisibleRootItem()
         return self.find_parent_index_by_id(root, id)
 
-    def get_children_index_by_id(self, parent_id, child_id):
+    def get_children_index_by_id(
+            self,
+            parent_id: int,
+            child_id: int
+    ) -> QModelIndex:
         """
         Return QModelIndex of child item based on its ID and parent ID.
 
@@ -775,7 +853,11 @@ class Ctx:
         parent_item = model.itemFromIndex(parent_index)
         return self.find_index_by_id(parent_item, child_id)
 
-    def find_child_index_by_id(self, root_item, child_id):
+    def find_child_index_by_id(
+            self,
+            root_item: QStandardItem,
+            child_id: int
+    ) -> QModelIndex:
         """
         Find and return QModelIndex of child based on its ID, recursively searching through the model.
 
@@ -792,7 +874,7 @@ class Ctx:
                 return child_index
         return QModelIndex()
 
-    def get_child_index_by_id(self, child_id):
+    def get_child_index_by_id(self, child_id: int) -> QModelIndex:
         """
         Return QModelIndex of child item based on its ID.
 
@@ -826,7 +908,12 @@ class Ctx:
         """Save visible ctx list items"""
         self.store_expanded_groups()
 
-    def move_to_group(self, meta_id, group_id, update: bool = True):
+    def move_to_group(
+            self,
+            meta_id: int,
+            group_id: int,
+            update: bool = True
+    ):
         """
         Move ctx to group
 
@@ -849,7 +936,10 @@ class Ctx:
         self.group_id = None
         self.update()
 
-    def new_group(self, meta_id = None):
+    def new_group(
+            self,
+            meta_id: Optional[int] = None
+    ):
         """
         Open new group dialog
 
@@ -861,7 +951,11 @@ class Ctx:
         self.window.ui.dialog['create'].show()
         self.window.ui.dialog['create'].input.setFocus()
 
-    def create_group(self, name: str = None, meta_id = None):
+    def create_group(
+            self,
+            name: Optional[str] = None,
+            meta_id: Optional[int] = None
+    ):
         """
         Make directory
 
@@ -888,7 +982,11 @@ class Ctx:
             self.select_group(id)
             self.group_id = id
 
-    def rename_group(self, id: int, force: bool = False):
+    def rename_group(
+            self,
+            id: int,
+            force: bool = False
+    ):
         """
         Rename group
 
@@ -904,7 +1002,12 @@ class Ctx:
             self.window.ui.dialog['rename'].current = id
             self.window.ui.dialog['rename'].show()
 
-    def update_group_name(self, id: int, name: str, close: bool = True):
+    def update_group_name(
+            self,
+            id: int,
+            name: str,
+            close: bool = True
+    ):
         """
         Update group name
 
@@ -945,7 +1048,11 @@ class Ctx:
         self.window.ui.nodes['ctx.list'].setCurrentIndex(index)
         self.window.ui.nodes['ctx.list'].unlocked = False  # tmp allow change if locked (disable)
 
-    def delete_group(self, id: int, force: bool = False):
+    def delete_group(
+            self,
+            id: int,
+            force: bool = False
+    ):
         """
         Delete group only
 
@@ -966,7 +1073,11 @@ class Ctx:
                 self.group_id = None
             self.update()
 
-    def delete_group_all(self, id: int, force: bool = False):
+    def delete_group_all(
+            self,
+            id: int,
+            force: bool = False
+    ):
         """
         Delete group with all items
 
