@@ -6,9 +6,11 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.07 23:00:00                  #
+# Updated Date: 2025.01.18 16:00:00                  #
 # ================================================== #
 
+from typing import Dict
+
 from pygpt_net.utils import trans
 
 
@@ -32,44 +34,68 @@ class Mapping:
         # nodes labels
         for k in self.mapping['nodes']:
             if k in self.window.ui.nodes:
-                self.window.ui.nodes[k].setText(trans(self.mapping['nodes'][k]))
+                try:
+                    self.window.ui.nodes[k].setText(trans(self.mapping['nodes'][k]))
+                except:
+                    pass
 
         # menu title
         for k in self.mapping['menu.title']:
             if k in self.window.ui.menu:
-                self.window.ui.menu[k].setTitle(trans(self.mapping['menu.title'][k]))
+                try:
+                    self.window.ui.menu[k].setTitle(trans(self.mapping['menu.title'][k]))
+                except:
+                    pass
 
         # menu text
         for k in self.mapping['menu.text']:
             if k in self.window.ui.menu:
-                self.window.ui.menu[k].setText(trans(self.mapping['menu.text'][k]))
+                try:
+                    self.window.ui.menu[k].setText(trans(self.mapping['menu.text'][k]))
+                except:
+                    pass
 
         # menu tooltip
         for k in self.mapping['menu.tooltip']:
             if k in self.window.ui.menu:
-                self.window.ui.menu[k].setToolTip(trans(self.mapping['menu.tooltip'][k]))
+                try:
+                    self.window.ui.menu[k].setToolTip(trans(self.mapping['menu.tooltip'][k]))
+                except:
+                    pass
 
         # dialog title
         for k in self.mapping['dialog.title']:
             if k in self.window.ui.dialog:
-                self.window.ui.dialog[k].setWindowTitle(trans(self.mapping['dialog.title'][k]))
+                try:
+                    self.window.ui.dialog[k].setWindowTitle(trans(self.mapping['dialog.title'][k]))
+                except:
+                    pass
 
         # tooltip
         for k in self.mapping['tooltip']:
             if k in self.window.ui.nodes:
-                self.window.ui.nodes[k].setToolTip(trans(self.mapping['tooltip'][k]))
+                try:
+                    self.window.ui.nodes[k].setToolTip(trans(self.mapping['tooltip'][k]))
+                except:
+                    pass
 
         # placeholder
         for k in self.mapping['placeholder']:
             if k in self.window.ui.nodes:
-                self.window.ui.nodes[k].setPlaceholderText(trans(self.mapping['placeholder'][k]))
+                try:
+                    self.window.ui.nodes[k].setPlaceholderText(trans(self.mapping['placeholder'][k]))
+                except:
+                    pass
 
         # menu tab tools
         for k in self.window.controller.tools.get_tab_tools():
             if k in self.window.ui.menu:
-                self.window.ui.menu[k].setText(trans("output.tab." + self.window.controller.tools.get_tab_tools()[k][0]))
+                try:
+                    self.window.ui.menu[k].setText(trans("output.tab." + self.window.controller.tools.get_tab_tools()[k][0]))
+                except:
+                    pass
 
-    def get_mapping(self) -> dict:
+    def get_mapping(self) -> Dict[str, Dict[str, str]]:
         """
         Get nodes => locale keys mapping
 
@@ -104,6 +130,8 @@ class Mapping:
         nodes['prompt.mode.label'] = 'toolbox.mode.label'
         nodes['prompt.model.label'] = 'toolbox.model.label'
         nodes['preset.presets.label'] = 'toolbox.presets.label'
+        nodes['preset.agents.label'] = 'toolbox.agents.label'
+        nodes['preset.experts.label'] = 'toolbox.experts.label'
         # nodes['preset.presets.new'] = 'preset.new'
         # nodes['preset.clear'] = 'preset.clear'
         nodes['preset.use'] = 'preset.use'
@@ -111,6 +139,13 @@ class Mapping:
         nodes['toolbox.prompt.label'] = 'toolbox.prompt'
         nodes["indexes.label"] = "toolbox.indexes.label"
         nodes["llama_index.mode.label"] = "toolbox.llama_index.mode.label"
+        nodes["agent.llama.loop.score.label"] = "toolbox.agent.llama.loop.score.label"
+        nodes["agent.llama.loop.label"] = "toolbox.agent.llama.loop.label"
+        nodes["agent.llama.loop.enabled"] = "toolbox.agent.llama.loop.enabled.label"
+        nodes["agent.iterations.label"] = "toolbox.agent.iterations.label"
+        nodes["agent.auto_stop"] = "toolbox.agent.auto_stop.label"
+        nodes["agent.continue"] = "toolbox.agent.continue.label"
+        nodes['layout.split'] = "layout.split"
         # nodes["indexes.new"] = "idx.new"
 
         # input
@@ -181,6 +216,12 @@ class Mapping:
         nodes['preset.user_name.label'] = 'preset.user_name'
         nodes['preset.temperature.label'] = 'preset.temperature'
         nodes['preset.prompt.label'] = 'preset.prompt'
+        nodes['preset.idx.label'] = 'preset.idx'
+        nodes['preset.agent_provider.label'] = 'preset.agent_provider'
+        nodes['preset.assistant_id.label'] = 'preset.assistant_id'
+        nodes['preset.tool.function.label.all'] = 'preset.tool.function.tip.all'
+        nodes['preset.tool.function.label.assistant'] = 'preset.tool.function.tip.assistant'
+        nodes['preset.tool.function.label.agent_llama'] = 'preset.tool.function.tip.agent_llama'
         nodes['preset.btn.current'] = 'dialog.preset.btn.current'
         nodes['preset.btn.save'] = 'dialog.preset.btn.save'
 
@@ -215,6 +256,13 @@ class Mapping:
         nodes['attachments_uploaded.btn.clear'] = 'attachments_uploaded.btn.clear'
         nodes['attachments_uploaded.sync.tip'] = 'attachments_uploaded.sync.tip'
 
+        nodes['tip.input.attachments.ctx'] = 'tip.input.attachments.ctx'
+        nodes['input.attachments.ctx.mode.label'] = 'attachments.ctx.label'
+        nodes['input.attachments.ctx.mode.full'] = 'attachments.ctx.mode.full'
+        nodes['input.attachments.ctx.mode.query'] = 'attachments.ctx.mode.query'
+        nodes['input.attachments.ctx.mode.query_summary'] = 'attachments.ctx.mode.summary'
+        nodes['input.attachments.ctx.mode.off'] = 'attachments.ctx.mode.off'
+
         # filesystem
         nodes['idx.btn.db.index_all'] = 'settings.llama.extra.btn.idx_db_all'
         nodes['idx.btn.db.index_update'] = 'settings.llama.extra.btn.idx_db_update'
@@ -290,7 +338,6 @@ class Mapping:
         nodes['tool.indexer.file.options.replace'] = 'tool.indexer.option.replace'
         nodes['tool.indexer.web.loader.label'] = 'tool.indexer.tab.web.loader'
         nodes['tool.indexer.web.options.label'] = 'tool.indexer.tab.web.source'
-        nodes['tool.indexer.web.config.label'] = 'tool.indexer.tab.web.cfg'
         nodes['tool.indexer.web.options.replace'] = 'tool.indexer.option.replace'
         nodes['tool.indexer.file.header.tip'] = 'tool.indexer.tab.files.tip'
         nodes['tool.indexer.web.header.tip'] = 'tool.indexer.tab.web.tip'
@@ -311,12 +358,14 @@ class Mapping:
         menu_title['theme.light'] = 'menu.theme.light'
         menu_title['theme.syntax'] = 'menu.theme.syntax'
         menu_title['theme.density'] = 'menu.theme.density'
+        menu_title['theme.style'] = 'menu.theme.style'
         menu_title['menu.plugins'] = 'menu.plugins'
         menu_title['menu.plugins.presets'] = 'menu.plugins.presets'
         menu_title['menu.about'] = 'menu.info'
         menu_title['menu.audio'] = 'menu.audio'
         menu_title['menu.video'] = 'menu.video'
         menu_title['menu.tools'] = 'menu.tools'
+        menu_title['menu.donate'] = 'menu.info.donate'
 
         # menu text
         menu_text = {}
@@ -336,7 +385,6 @@ class Mapping:
         menu_text['config.profile.new'] = 'menu.config.profile.new'
         menu_text['config.save'] = 'menu.config.save'
         menu_text['theme.tooltips'] = 'menu.theme.tooltips'
-        menu_text['theme.blocks'] = 'menu.theme.blocks'
         menu_text['theme.settings'] = 'menu.theme.settings'
         menu_text['plugins.presets.new'] = 'menu.plugins.presets.new'
         menu_text['plugins.presets.edit'] = 'menu.plugins.presets.edit'
@@ -347,9 +395,9 @@ class Mapping:
         menu_text['info.docs'] = 'menu.info.docs'
         menu_text['info.pypi'] = 'menu.info.pypi'
         menu_text['info.snap'] = 'menu.info.snap'
-        menu_text['info.donate'] = 'menu.info.donate'
         menu_text['info.website'] = 'menu.info.website'
         menu_text['info.github'] = 'menu.info.github'
+        menu_text['info.report'] = 'menu.info.report'
         menu_text['audio.output'] = 'menu.audio.output'
         menu_text['audio.input'] = 'menu.audio.input'
         menu_text['audio.control.plugin'] = 'menu.audio.control.plugin'
@@ -407,7 +455,7 @@ class Mapping:
         tooltips['icon.audio.output'] = 'icon.audio.output'
         tooltips['icon.audio.input'] = 'icon.audio.input'
         tooltips['assistant.store.btn.refresh_status'] = 'dialog.assistant.store.btn.refresh_status'
-        tooltips['inline.vision'] = 'vision.checkbox.tooltip'
+        tooltips['agent.llama.loop.score'] = 'toolbox.agent.llama.loop.score.tooltip'
 
         # menu tooltips
         menu_tooltips = {}
