@@ -6,15 +6,24 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.05.03 12:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
 import re
+from typing import Optional, List, Dict
 
 from PySide6.QtGui import QTextCursor
+from PySide6.QtWidgets import QTextEdit
 
+from pygpt_net.core.types import (
+    MODE_AGENT,
+    MODE_ASSISTANT,
+    MODE_CHAT,
+    MODE_EXPERT,
+)
 from pygpt_net.controller.presets.editor import Editor
-from pygpt_net.core.access.events import AppEvent
+from pygpt_net.core.events import AppEvent
+from pygpt_net.item.preset import PresetItem
 from pygpt_net.utils import trans
 
 
@@ -47,7 +56,21 @@ class Presets:
 
         # update all layout
         self.window.controller.ui.update()
-        self.window.core.dispatcher.dispatch(AppEvent(AppEvent.PRESET_SELECTED))  # app event
+        self.window.controller.model.select_current()
+        self.window.dispatch(AppEvent(AppEvent.PRESET_SELECTED))  # app event
+
+    def get_current(self) -> Optional[PresetItem]:
+        """
+        Get current preset
+
+        :return: preset item
+        """
+        preset_id = self.window.core.config.get('preset')
+        mode = self.window.core.config.get('mode')
+        if preset_id is not None and preset_id != "":
+            if self.window.core.presets.has(mode, preset_id):
+                return self.window.core.presets.get_by_id(mode, preset_id)
+        return None
 
     def next(self):
         """Select next preset"""
@@ -71,7 +94,11 @@ class Presets:
             self.window.ui.nodes['preset.prompt'].toPlainText()
         )
 
-    def paste_prompt(self, idx: int, parent: str = "global"):
+    def paste_prompt(
+            self,
+            idx: int,
+            parent: str = "global"
+    ):
         """
         Paste prompt from template
 
@@ -88,7 +115,11 @@ class Presets:
         elif parent == "editor":
             self.paste_to_textarea(self.window.ui.config["preset"]["prompt"], template['prompt'])
 
-    def paste_custom_prompt(self, idx: int, parent: str = "global"):
+    def paste_custom_prompt(
+            self,
+            idx: int,
+            parent: str = "global"
+    ):
         """
         Paste prompt from custom template
 
@@ -105,7 +136,11 @@ class Presets:
         elif parent == "editor":
             self.paste_to_textarea(self.window.ui.config["preset"]["prompt"], template.content)
 
-    def save_prompt(self, name: str = "", force: bool = False):
+    def save_prompt(
+            self,
+            name: str = "",
+            force: bool = False
+    ):
         """
         Save prompt to file
 
@@ -114,7 +149,7 @@ class Presets:
         """
         content = self.window.ui.nodes['input'].toPlainText()
         if content.strip() == "":
-            self.window.ui.status("Prompt is empty!")
+            self.window.update_status("Prompt is empty!")
             return
         if not force:
             self.window.ui.dialog['rename'].id = 'prompt.custom.new'
@@ -124,9 +159,14 @@ class Presets:
             return
         self.window.ui.dialog['rename'].close()
         self.window.core.prompt.custom.new(name, content)
-        self.window.ui.status("Prompt saved")
+        self.window.update_status("Prompt saved")
 
-    def rename_prompt(self, uuid: str, name: str = "", force: bool = False):
+    def rename_prompt(
+            self,
+            uuid: str,
+            name: str = "",
+            force: bool = False
+    ):
         """
         Rename prompt
 
@@ -146,9 +186,13 @@ class Presets:
         self.window.ui.dialog['rename'].close()
         item.name = name
         self.window.core.prompt.custom.save()
-        self.window.ui.status("Prompt renamed")
+        self.window.update_status("Prompt renamed")
 
-    def delete_prompt(self, uuid: str, force: bool = False):
+    def delete_prompt(
+            self,
+            uuid: str,
+            force: bool = False
+    ):
         """
         Delete prompt
 
@@ -166,9 +210,13 @@ class Presets:
             return
         self.window.ui.dialog['confirm'].close()
         self.window.core.prompt.custom.delete(uuid)
-        self.window.ui.status("Prompt deleted")
+        self.window.update_status("Prompt deleted")
 
-    def paste_to_textarea(self, textarea, text: str):
+    def paste_to_textarea(
+            self,
+            textarea: QTextEdit,
+            text: str
+    ):
         """
         Paste text to textarea
 
@@ -192,7 +240,11 @@ class Presets:
         textarea.setTextCursor(cur)
         textarea.setFocus()
 
-    def set(self, mode: str, preset_id: str):
+    def set(
+            self,
+            mode: str,
+            preset_id: str
+    ):
         """
         Set preset
 
@@ -206,7 +258,11 @@ class Presets:
             self.window.core.config.data['current_preset'] = {}
         self.window.core.config.data['current_preset'][mode] = preset_id
 
-    def set_by_idx(self, mode: str, idx: int):
+    def set_by_idx(
+            self,
+            mode: str,
+            idx: int
+    ):
         """
         Set preset by index
 
@@ -276,6 +332,8 @@ class Presets:
         self.window.core.config.set('prompt', preset.prompt)
         self.window.core.config.set('ai_name', preset.ai_name)
         self.window.core.config.set('user_name', preset.user_name)
+        self.window.core.config.set('agent.llama.provider', preset.agent_provider)
+        self.window.core.config.set('agent.llama.idx', preset.idx)
 
     def update_current(self):
         """Update current mode, model and preset"""
@@ -290,6 +348,8 @@ class Presets:
                 self.window.core.config.set('ai_name', preset.ai_name)
                 self.window.core.config.set('prompt', preset.prompt)
                 self.window.core.config.set('temperature', preset.temperature)
+                self.window.core.config.set('agent.llama.provider', preset.agent_provider)
+                self.window.core.config.set('agent.llama.idx', preset.idx)
                 return
 
         self.window.core.config.set('user_name', None)
@@ -297,12 +357,12 @@ class Presets:
         self.window.core.config.set('temperature', 1.0)
 
         # set default prompt if mode is chat
-        if mode == 'chat':
+        if mode == MODE_CHAT:
             self.window.core.config.set('prompt', self.window.core.prompt.get('default'))
         else:
             self.window.core.config.set('prompt', None)
 
-    def get_current_functions(self) -> list or None:
+    def get_current_functions(self) -> Optional[List[Dict]]:
         """
         Get current preset functions
 
@@ -340,13 +400,13 @@ class Presets:
                                 and self.window.core.models.is_allowed(preset.model, mode):
                             self.window.core.config.set('model', preset.model)
                             self.window.controller.model.set(mode, preset.model)
-                            self.window.controller.model.update_list()
+                            self.window.controller.model.init_list()
                             self.window.controller.model.select_current()
 
     def refresh(self):
         """Refresh presets"""
         mode = self.window.core.config.get('mode')
-        if mode == 'assistant':
+        if mode == MODE_ASSISTANT:
             return
 
         self.select_default()  # if no preset then select previous from current presets
@@ -379,7 +439,7 @@ class Presets:
         filename = re.sub(r'[^a-zA-Z0-9_\-\.]', '_', filename)
         return filename
 
-    def duplicate(self, idx: int = None):
+    def duplicate(self, idx: Optional[int] = None):
         """
         Duplicate preset
 
@@ -395,9 +455,9 @@ class Presets:
                     self.refresh()
                     idx = self.window.core.presets.get_idx_by_id(mode, new_id)
                     self.editor.edit(idx)
-                    self.window.ui.status(trans('status.preset.duplicated'))
+                    self.window.update_status(trans('status.preset.duplicated'))
 
-    def enable(self, idx: int = None):
+    def enable(self, idx: Optional[int] = None):
         """
         Enable preset
 
@@ -411,7 +471,7 @@ class Presets:
                     self.window.core.presets.enable(preset_id)
                     self.refresh()
 
-    def disable(self, idx: int = None):
+    def disable(self, idx: Optional[int] = None):
         """
         Disable preset
 
@@ -454,14 +514,18 @@ class Presets:
                 self.window.core.presets.items[preset].temperature = 1.0
                 self.refresh()
 
-        self.window.ui.status(trans('status.preset.cleared'))
+        self.window.update_status(trans('status.preset.cleared'))
 
         # reload assistant default instructions
         mode = self.window.core.config.get('mode')
-        if mode == "assistant":
+        if mode == MODE_ASSISTANT:
             self.window.core.assistants.load()
 
-    def delete(self, idx: int = None, force: bool = False):
+    def delete(
+            self,
+            idx: Optional[int] = None,
+            force: bool = False
+    ):
         """
         Delete preset
 
@@ -487,7 +551,7 @@ class Presets:
                         self.window.ui.nodes['preset.prompt'].setPlainText("")
                     self.window.core.presets.remove(preset_id, True)
                     self.refresh()
-                    self.window.ui.status(trans('status.preset.deleted'))
+                    self.window.update_status(trans('status.preset.deleted'))
 
     def restore(self, force: bool = False):
         """
@@ -503,12 +567,12 @@ class Presets:
             )
             return
         mode = self.window.core.config.get('mode')
-        if mode == "agent":
-            mode = "expert"  # shared presets
+        if mode == MODE_AGENT:
+            mode = MODE_EXPERT  # shared presets
         self.window.core.presets.restore(mode)
         self.refresh()
 
-    def is_current(self, idx: int = None) -> bool:
+    def is_current(self, idx: Optional[int] = None) -> bool:
         """
         Check if preset is current
 
@@ -517,8 +581,8 @@ class Presets:
         """
         if idx is not None:
             mode = self.window.core.config.get('mode')
-            if mode == "agent":
-                mode = "expert"  # shared presets
+            if mode == MODE_AGENT:
+                mode = MODE_EXPERT  # shared presets
             preset_id = self.window.core.presets.get_by_idx(idx, mode)
             if preset_id is not None and preset_id != "":
                 if preset_id == "current." + mode:
@@ -544,7 +608,7 @@ class Presets:
         # if self.window.controller.chat.input.generating:
         # return True
         mode = self.window.core.config.get('mode')
-        if mode == "assistant":
+        if mode == MODE_ASSISTANT:
             return True
         return False
 
