@@ -6,16 +6,30 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.05.01 17:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
 import datetime
 import os
+from typing import Any, Optional, Dict
 
+from pygpt_net.core.types import (
+    MODE_AGENT,
+    MODE_AGENT_LLAMA,
+    MODE_ASSISTANT,
+    MODE_AUDIO,
+    MODE_CHAT,
+    MODE_COMPLETION,
+    MODE_EXPERT,
+    MODE_LANGCHAIN,
+    MODE_LLAMA_INDEX,
+    MODE_VISION, 
+    MODE_IMAGE,
+)
 from pygpt_net.item.preset import PresetItem
 from pygpt_net.utils import trans
-from .experts import Experts
 
+from .experts import Experts
 
 class Editor:
     def __init__(self, window=None):
@@ -43,39 +57,47 @@ class Editor:
                 "type": "text",
                 "label": "preset.user_name",
             },
-            "img": {
+            MODE_IMAGE: {
                 "type": "bool",
                 "label": "preset.img",
             },
-            "chat": {
+            MODE_CHAT: {
                 "type": "bool",
                 "label": "preset.chat",
             },
-            "completion": {
+            MODE_COMPLETION: {
                 "type": "bool",
                 "label": "preset.completion",
             },
-            "vision": {
+            MODE_VISION: {
                 "type": "bool",
                 "label": "preset.vision",
             },
-            "langchain": {
+            MODE_LANGCHAIN: {
                 "type": "bool",
                 "label": "preset.langchain",
             },
-            "expert": {
+            MODE_EXPERT: {
                 "type": "bool",
                 "label": "preset.expert",
             },
-            "agent": {
+            MODE_AGENT_LLAMA: {
+                "type": "bool",
+                "label": "preset.agent_llama",
+            },
+            MODE_AGENT: {
                 "type": "bool",
                 "label": "preset.agent",
             },
+            MODE_AUDIO: {
+                "type": "bool",
+                "label": "preset.audio",
+            },
             # "assistant": {
             # "type": "bool",
             # "label": "preset.assistant",
             # },
-            "llama_index": {
+            MODE_LLAMA_INDEX: {
                 "type": "bool",
                 "label": "preset.llama_index",
             },
@@ -99,6 +121,23 @@ class Editor:
                 "label": "preset.prompt",
                 "context_options": ["prompt.template.paste"],
             },
+            "idx": {
+                "type": "combo",
+                "label": "preset.idx",
+                "description": "preset.idx.desc",
+                "use": "idx",
+            },
+            "agent_provider": {
+                "type": "combo",
+                "label": "preset.agent_provider",
+                "description": "preset.agent_provider.desc",
+                "use": "agent_provider",
+            },
+            "assistant_id": {
+                "type": "text",
+                "label": "preset.assistant_id",
+                "description": "preset.assistant_id.desc",
+            },
             "tool.function": {
                 "type": "dict",
                 "label": "preset.tool.function",
@@ -114,10 +153,14 @@ class Editor:
                 },
             },
         }
+        self.hidden_by_mode = {  # hidden fields by mode
+            MODE_CHAT: ["idx"],
+            MODE_AGENT_LLAMA: ["temperature"],
+        }
         self.id = "preset"
         self.current = None
 
-    def get_options(self) -> dict:
+    def get_options(self) -> Dict[str, Dict[str, Any]]:
         """
         Get preset options
 
@@ -125,7 +168,7 @@ class Editor:
         """
         return self.options
 
-    def get_option(self, id: str) -> dict:
+    def get_option(self, id: str) -> Dict[str, Any]:
         """
         Get preset option
 
@@ -134,13 +177,32 @@ class Editor:
         """
         return self.options[id]
 
+    def show_hide_by_mode(self):
+        """Show or hide fields by mode"""
+        # TODO: implement this!
+        return
+        mode = self.window.core.config.get('mode')
+        for key in self.options:
+            if mode in self.hidden_by_mode:
+                if key in self.hidden_by_mode[mode]:
+                    self.window.ui.config[self.id][key].setVisible(False)
+                    label = 'preset.' + key + '.label'
+                    if label in self.window.ui.nodes:
+                        self.window.ui.nodes[label].setVisible(False)
+                else:
+                    self.window.ui.config[self.id][key].setVisible(True)
+                    label = 'preset.' + key + '.label'
+                    if label in self.window.ui.nodes:
+                        self.window.ui.nodes[label].setVisible(True)
+
     def setup(self):
-        """
-        Setup preset editor
-        """
+        """Setup preset editor"""
+        # update after agents register
+        self.window.ui.config[self.id]['agent_provider'].set_keys(
+            self.window.controller.config.placeholder.apply_by_id('agent_provider')
+        )
         # add hooks for config update in real-time
         self.window.ui.add_hook("update.preset.prompt", self.hook_update)
-
         # register functions dictionary
         parent = "preset"
         key = "tool.function"
@@ -150,20 +212,32 @@ class Editor:
             self.get_option(key),
         )
 
-    def hook_update(self, key, value, caller, *args, **kwargs):
+    def hook_update(
+            self,
+            key: str,
+            value: Any,
+            caller,
+            *args,
+            **kwargs
+    ):
         """
         Hook: on settings update in real-time (prompt)
+
+        :param key: setting key
+        :param value: setting value
+        :param caller: caller id
+        :param args: additional arguments
+        :param kwargs: additional keyword arguments
         """
         mode = self.window.core.config.get('mode')
         if key == "prompt":
             self.window.core.config.set('prompt', value)
-            if mode == 'assistant':
+            if mode == MODE_ASSISTANT:
                 self.window.controller.assistant.from_global()  # update current assistant, never called!!!!!
             else:
                 self.window.controller.presets.from_global()  # update current preset
 
-
-    def edit(self, idx: int = None):
+    def edit(self, idx: Optional[int] = None):
         """
         Open preset editor
 
@@ -176,7 +250,7 @@ class Editor:
         self.init(preset)
         self.window.ui.dialogs.open_editor('editor.preset.presets', idx, width=800)
 
-    def init(self, id: str = None):
+    def init(self, id: Optional[str] = None):
         """
         Initialize preset editor
 
@@ -206,24 +280,28 @@ class Editor:
         # set current mode at start
         if id is None:
             mode = self.window.core.config.get("mode")
-            if mode == "chat":
+            if mode == MODE_CHAT:
                 data.chat = True
-            elif mode == "completion":
+            elif mode == MODE_COMPLETION:
                 data.completion = True
-            elif mode == "img":
+            elif mode == MODE_IMAGE:
                 data.img = True
-            elif mode == "vision":
+            elif mode == MODE_VISION:
                 data.vision = True
-            elif mode == "langchain":
+            elif mode == MODE_LANGCHAIN:
                 data.langchain = True
-            # elif mode == "'"assistant':
+            # elif mode == MODE_ASSISTANT:
                 # data.assistant = True
-            elif mode == "llama_index":
+            elif mode == MODE_LLAMA_INDEX:
                 data.llama_index = True
-            elif mode == "expert":
+            elif mode == MODE_EXPERT:
                 data.expert = True
-            elif mode == "agent":
+            elif mode == MODE_AGENT:
                 data.agent = True
+            elif mode == MODE_AGENT_LLAMA:
+                data.agent_llama = True
+            elif mode == MODE_AUDIO:
+                data.audio = True
 
         options = {}
         data_dict = data.to_dict()
@@ -259,9 +337,18 @@ class Editor:
             self.window.ui.config[self.id]['tool.function'].model.updateData([])
 
         # set focus to name field
+        current_model = self.window.core.config.get('model')
+        # set current model in combo box as selected
+        if id is None:
+            self.window.ui.config[self.id]['model'].set_value(current_model)
         self.window.ui.config[self.id]['name'].setFocus()
+        self.show_hide_by_mode()
 
-    def save(self, force: bool = False, close: bool = True):
+    def save(
+            self,
+            force: bool = False,
+            close: bool = True
+    ):
         """
         Save ore create preset
 
@@ -274,7 +361,17 @@ class Editor:
             option=self.options["filename"],
         )
         mode = self.window.core.config.get("mode")
-        modes = ["chat", "completion", "img", "vision", "langchain", "llama_index", "expert"]
+        modes = [
+            MODE_CHAT,
+            MODE_COMPLETION,
+            MODE_IMAGE,
+            MODE_VISION,
+            MODE_LANGCHAIN,
+            MODE_LLAMA_INDEX,
+            MODE_EXPERT,
+            MODE_AGENT_LLAMA,
+            MODE_AUDIO,
+        ]
 
         # disallow editing default preset
         if id == "current." + mode:
@@ -289,7 +386,7 @@ class Editor:
             )
             if name is None or name == "":
                 self.window.ui.dialogs.alert(trans('alert.preset.empty_id'))
-                self.window.ui.status(trans('status.preset.empty_id'))
+                self.window.update_status(trans('status.preset.empty_id'))
                 return
 
             # generate new filename
@@ -324,7 +421,7 @@ class Editor:
             ):
                 is_mode = True
                 break
-        if mode != "agent" and not is_mode:
+        if mode != MODE_AGENT and not is_mode:
             self.window.ui.dialogs.alert(
                 trans('alert.preset.no_chat_completion')
             )
@@ -334,8 +431,8 @@ class Editor:
         self.assign_data(id)
 
         # if agent, assign experts and select only agent mode
-        if self.window.core.config.get('mode') == 'agent':
-            self.window.core.presets.items[id].mode = ["agent"]
+        if self.window.core.config.get('mode') == MODE_AGENT:
+            self.window.core.presets.items[id].mode = [MODE_AGENT]
 
         # apply changes to current active preset
         current = self.window.core.config.get('preset')
@@ -361,7 +458,7 @@ class Editor:
                 option=self.options["filename"],
                 value=id,
             )
-        self.window.ui.status(trans('status.preset.saved'))
+        self.window.update_status(trans('status.preset.saved'))
 
         # sort by name
         self.window.core.presets.sort_by_name()
@@ -473,8 +570,13 @@ class Editor:
             value=self.window.core.config.get('model'),
         )
 
-    def update_from_global(self, key, value):
-        """Update field from global config"""
+    def update_from_global(self, key: str, value: Any):
+        """
+        Update field from global config
+
+        :param key: field key
+        :param value: field value
+        """
         preset_id = self.window.core.config.get('preset')
         if preset_id is not None and preset_id != "":
             if preset_id in self.window.core.presets.items:
