@@ -6,12 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.04.29 07:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
 import copy
 import os
 from pathlib import Path
+from typing import Optional, Dict, Any
 from uuid import uuid4
 
 from PySide6.QtGui import QAction
@@ -40,7 +41,12 @@ class Profile:
             self.window.profiles.setup()  # widget dialog
             self.dialog_initialized = True
 
-    def switch(self, uuid: str, force: bool = False, save_current: bool = True):
+    def switch(
+            self,
+            uuid: str,
+            force: bool = False,
+            save_current: bool = True
+    ):
         """
         Switch profile
 
@@ -56,7 +62,7 @@ class Profile:
         if profile is None:
             self.window.ui.dialogs.alert("Profile not found!")
             return
-        self.window.ui.status("Please wait...")
+        self.window.update_status("Please wait...")
 
         if save_current:
             print("Saving all settings in current profile...")
@@ -72,7 +78,7 @@ class Profile:
         self.update_menu()
         self.update_list()
         self.window.ui.update_title()
-        self.window.ui.status(trans("dialog.profile.status.changed") + ": " + profile['name'])
+        self.window.update_status(trans("dialog.profile.status.changed") + ": " + profile['name'])
         self.select_current_on_list()
 
     def select_current_on_list(self):
@@ -87,7 +93,7 @@ class Profile:
                 break
             idx += 1
 
-    def get_profiles(self) -> dict:
+    def get_profiles(self) -> Dict[str, Dict[str, Any]]:
         """
         Get profiles
 
@@ -158,7 +164,13 @@ class Profile:
         else:
             self.open()
 
-    def handle_update(self, mode: str, name: str, path: str, uuid: str = None):
+    def handle_update(
+            self,
+            mode: str,
+            name: str,
+            path: str,
+            uuid: Optional[str] = None
+    ):
         """
         Handle new/edit profile dialog
 
@@ -186,7 +198,7 @@ class Profile:
         if mode == 'create':
             # create new profile
             uuid = self.window.core.config.profile.add(name, path)
-            self.window.ui.status(trans("dialog.profile.status.created"))
+            self.window.update_status(trans("dialog.profile.status.created"))
             if self.window.ui.nodes['dialog.profile.checkbox.switch'].isChecked():
                 self.switch(uuid, force=True)
 
@@ -195,7 +207,7 @@ class Profile:
             profile = self.window.core.config.profile.get(uuid)
             old_path = profile['workdir'].replace("%HOME%", str(Path.home()))
             self.window.core.config.profile.update_profile(uuid, name, path)
-            self.window.ui.status(trans("dialog.profile.status.updated"))
+            self.window.update_status(trans("dialog.profile.status.updated"))
 
             # if current profile and path was changed then reload:
             if uuid == current:
@@ -239,7 +251,7 @@ class Profile:
 
             # make duplicate
             self.duplicate(uuid, name, path)
-            self.window.ui.status(trans("dialog.profile.status.duplicated"))
+            self.window.update_status(trans("dialog.profile.status.duplicated"))
             if self.window.ui.nodes['dialog.profile.checkbox.switch'].isChecked():
                 self.switch(uuid, force=True)
 
@@ -303,7 +315,7 @@ class Profile:
             name = profile['name']
             # remove profile
             if self.window.core.config.profile.remove(uuid):
-                self.window.ui.status(trans("dialog.profile.status.removed") + ": " + name)
+                self.window.update_status(trans("dialog.profile.status.removed") + ": " + name)
                 self.update_list()
                 self.update_menu()
 
@@ -352,11 +364,16 @@ class Profile:
                     remove_db=remove_db,
                     remove_datadir=remove_datadir,
                 )
-                self.window.ui.status(trans("dialog.profile.status.deleted") + ": " + name)
+                self.window.update_status(trans("dialog.profile.status.deleted") + ": " + name)
                 self.update_list()
                 self.update_menu()
 
-    def duplicate(self, uuid: str, new_name: str, new_path: str):
+    def duplicate(
+            self,
+            uuid: str,
+            new_name: str,
+            new_path: str
+    ):
         """
         Duplicate profile
 
@@ -384,7 +401,7 @@ class Profile:
         path_from = profile['workdir'].replace("%HOME%", str(Path.home()))
         path_to = new_path
         print("Copying all files from {} to: {}".format(path_from, path_to))
-        self.window.ui.status("Copying files...")
+        self.window.update_status("Copying files...")
         result = self.window.core.filesystem.copy_workdir(
             path_from,
             path_to,
@@ -393,10 +410,10 @@ class Profile:
         )
         if not result:
             self.window.ui.dialogs.alert("Error copying files!")
-            self.window.ui.status("Error copying files!")
+            self.window.update_status("Error copying files!")
             return
         print("[OK] All files copied successfully.")
-        self.window.ui.status("Files copied.")
+        self.window.update_status("Files copied.")
         self.update_list()
         self.update_menu()
 
@@ -446,7 +463,7 @@ class Profile:
             )
             if uuid == current:
                 self.switch(uuid, force=True, save_current=False)  # reload current profile
-            self.window.ui.status("Profile cleared: " + profile['name'])
+            self.window.update_status("Profile cleared: " + profile['name'])
 
     def reset_by_idx(self, idx: int, force: bool = False):
         """
