@@ -6,11 +6,20 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.05 23:00:00                  #
+# Updated Date: 2024.12.14 19:00:00                  #
 # ================================================== #
 
+from pygpt_net.core.types import (
+    MODE_AGENT,
+    MODE_AGENT_LLAMA,
+    MODE_ASSISTANT,
+    MODE_EXPERT,
+    MODE_IMAGE,
+    MODE_LLAMA_INDEX,
+    MODE_VISION,
+)
 from pygpt_net.core.tabs.tab import Tab
-from pygpt_net.core.dispatcher import Event
+from pygpt_net.core.events import Event
 from pygpt_net.utils import trans
 
 
@@ -29,17 +38,21 @@ class Mode:
         mode = self.window.core.config.data['mode']
 
         # presets
-        if mode != "assistant":
+        if mode != MODE_ASSISTANT:
             self.window.ui.nodes['presets.widget'].setVisible(True)
         else:
             self.window.ui.nodes['presets.widget'].setVisible(False)
 
         # presets: labels
-        if mode == "agent":
+        if mode == MODE_AGENT:
             self.window.ui.nodes['preset.agents.label'].setVisible(True)
             self.window.ui.nodes['preset.experts.label'].setVisible(False)
             self.window.ui.nodes['preset.presets.label'].setVisible(False)
-        elif mode == "expert":
+        elif mode == MODE_AGENT_LLAMA:
+            self.window.ui.nodes['preset.agents.label'].setVisible(True)
+            self.window.ui.nodes['preset.experts.label'].setVisible(False)
+            self.window.ui.nodes['preset.presets.label'].setVisible(False)
+        elif mode == MODE_EXPERT:
             self.window.ui.nodes['preset.agents.label'].setVisible(False)
             self.window.ui.nodes['preset.experts.label'].setVisible(True)
             self.window.ui.nodes['preset.presets.label'].setVisible(False)
@@ -49,52 +62,97 @@ class Mode:
             self.window.ui.nodes['preset.presets.label'].setVisible(True)
 
         # presets: editor
-        if mode == "agent":
+        if mode == MODE_AGENT:
+            self.window.ui.nodes['preset.editor.user_name'].setVisible(True)
+            self.window.ui.nodes['preset.editor.temperature'].setVisible(True)
+            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(False)
             self.window.ui.nodes['preset.editor.functions'].setVisible(False)
             self.window.ui.nodes['preset.editor.modes'].setVisible(False)
             self.window.ui.nodes['preset.editor.experts'].setVisible(True)
             self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt.agent"))
+            self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
+            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
+            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(False)
+        elif mode == MODE_AGENT_LLAMA:
+            self.window.ui.nodes['preset.editor.user_name'].setVisible(False)
+            self.window.ui.nodes['preset.editor.temperature'].setVisible(False)
+            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(True)
+            self.window.ui.nodes['preset.editor.functions'].setVisible(False)
+            self.window.ui.nodes['preset.editor.modes'].setVisible(False)
+            self.window.ui.nodes['preset.editor.experts'].setVisible(False)
+            self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt.agent_llama"))
+            self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
+            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
+            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(True)
         else:
+            self.window.ui.nodes['preset.editor.user_name'].setVisible(True)
+            self.window.ui.nodes['preset.editor.temperature'].setVisible(True)
+            self.window.ui.nodes['preset.editor.agent_llama'].setVisible(False)
             self.window.ui.nodes['preset.editor.functions'].setVisible(True)
             self.window.ui.nodes['preset.editor.modes'].setVisible(True)
             self.window.ui.nodes['preset.editor.experts'].setVisible(False)
             self.window.ui.nodes["preset.prompt.label"].setText(trans("preset.prompt"))
+            self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
+            self.window.ui.nodes['preset.tool.function.label.agent_llama'].setVisible(False)
+
+            if mode == MODE_ASSISTANT:
+                self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(True)
+                self.window.ui.nodes['preset.tool.function.label.all'].setVisible(False)
+            else:
+                self.window.ui.nodes['preset.tool.function.label.assistant'].setVisible(False)
+                self.window.ui.nodes['preset.tool.function.label.all'].setVisible(True)
 
         # presets: clear
-        self.window.ui.nodes['preset.clear'].setVisible(False)
         """
-        if mode in ["img", "assistant"]:
+        self.window.ui.nodes['preset.clear'].setVisible(False)
+        if mode in [MODE_IMAGE, MODE_ASSISTANT]:
             self.window.ui.nodes['preset.clear'].setVisible(False)
         else:
             self.window.ui.nodes['preset.clear'].setVisible(True)
         """
 
         # presets: use
-        if mode == 'img':
+        """
+        if mode == MODE_IMAGE:
             self.window.ui.nodes['preset.use'].setVisible(True)
         else:
             self.window.ui.nodes['preset.use'].setVisible(False)
+        """
 
         # img options
-        if mode == "img":
+        if mode == MODE_IMAGE:
             self.window.ui.nodes['dalle.options'].setVisible(True)
         else:
             self.window.ui.nodes['dalle.options'].setVisible(False)
 
         # agent options
-        if mode == "agent":
+        if mode in [MODE_AGENT]:
             self.window.ui.nodes['agent.options'].setVisible(True)
         else:
             self.window.ui.nodes['agent.options'].setVisible(False)
 
+        # agent llama options
+        if mode in [MODE_AGENT_LLAMA]:
+            self.window.ui.nodes['agent_llama.options'].setVisible(True)
+        else:
+            self.window.ui.nodes['agent_llama.options'].setVisible(False)
+
+        """
+        # agent llama sys prompt
+        if mode in [MODE_AGENT_LLAMA]:
+            self.window.ui.nodes['preset.prompt'].setVisible(False)
+        else:
+            self.window.ui.nodes['preset.prompt'].setVisible(True)
+        """
+
         # assistants list
-        if mode == "assistant":
+        if mode == MODE_ASSISTANT:
             self.window.ui.nodes['assistants.widget'].setVisible(True)
         else:
             self.window.ui.nodes['assistants.widget'].setVisible(False)
 
         # indexes list
-        if mode == "llama_index":
+        if mode == MODE_LLAMA_INDEX:
             # self.window.ui.nodes['indexes.widget'].setVisible(True)
             self.window.ui.nodes['idx.options'].setVisible(True)
         else:
@@ -102,7 +160,8 @@ class Mode:
             self.window.ui.nodes['idx.options'].setVisible(False)
 
         # stream mode
-        if mode in ["img"]:
+        if mode in [MODE_IMAGE]:
+
             self.window.ui.nodes['input.stream'].setVisible(False)
         else:
             self.window.ui.nodes['input.stream'].setVisible(True)
@@ -119,13 +178,23 @@ class Mode:
         self.window.ui.tabs['input'].setTabVisible(1, show)  # attachments
 
         # uploaded files
-        if mode == "assistant":
+        if mode == MODE_ASSISTANT:
             self.window.ui.tabs['input'].setTabVisible(2, True)
+            self.window.ui.tabs['input'].setTabVisible(3, False)
         else:
-            self.window.ui.tabs['input'].setTabVisible(2, False)
+            if mode != MODE_IMAGE:
+                self.window.ui.tabs['input'].setTabVisible(2, False)
+                self.window.ui.tabs['input'].setTabVisible(3, True)
+            else:
+                self.window.ui.tabs['input'].setTabVisible(2, False)
+                self.window.ui.tabs['input'].setTabVisible(3, False)
 
-        # toggle chat footer
-        if not self.is_chat_tab():
+        # chat footer
+        self.toggle_chat_footer()
+
+    def toggle_chat_footer(self):
+        """Toggle chat footer"""
+        if self.window.controller.ui.tabs.get_current_type() != Tab.TAB_CHAT:
             self.hide_chat_footer()
         else:
             self.show_chat_footer()
@@ -135,18 +204,12 @@ class Mode:
         Check if vision is allowed
 
         :param mode: current mode
+        :return: True if vision is allowed
         """
-        if mode == "vision":
+        if self.window.controller.ui.vision.has_vision():
             return True
 
-        allowed = self.window.controller.painter.is_active()
-        if allowed:
-            return True
-
-        if mode in ["img", "assistant"]:
-            return False
-
-        if self.window.controller.ui.vision.is_vision_model():
+        if self.window.controller.painter.is_active():
             return True
 
         # event: UI: vision
@@ -155,33 +218,34 @@ class Mode:
             'mode': mode,
             'value': value,
         })
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
         return event.data['value']
 
     def are_attachments(self, mode: str) -> bool:
-        """Check if attachments are allowed"""
-        if mode in ["vision", "assistant"]:
+        """
+        Check if attachments are allowed
+
+        :param mode: current mode
+        :return: True if attachments are allowed
+        """
+        if mode in [MODE_VISION, MODE_ASSISTANT]:
             return True
 
-        if mode == "img":
+        if mode in [MODE_IMAGE]:
             return False
 
-        if self.window.controller.ui.vision.is_vision_model():
+        if self.window.controller.ui.vision.has_vision():
             return True
 
         # event: UI: attachments
-        value = False
+        # value = False
         event = Event(Event.UI_ATTACHMENTS, {
             'mode': mode,
-            'value': value,
+            'value': True,
         })
-        self.window.core.dispatcher.dispatch(event)
+        self.window.dispatch(event)
         return event.data['value']
 
-    def is_chat_tab(self) -> bool:
-        """Check if current tab is chat"""
-        return self.window.controller.ui.tabs.get_current_type() == Tab.TAB_CHAT
-
     def show_chat_footer(self):
         """Show chat footer"""
         self.window.ui.nodes['chat.footer'].setVisible(True)
