@@ -6,10 +6,12 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.04.17 01:00:00                  #
+# Updated Date: 2025.01.16 01:00:00                  #
 # ================================================== #
 
 import datetime
+from typing import Optional, Tuple, List, Dict, Any
+
 from packaging.version import Version
 
 from pygpt_net.item.index import IndexItem
@@ -22,6 +24,7 @@ from .indexing import Indexing
 from .llm import Llm
 from .chat import Chat
 from .metadata import Metadata
+from .ui import UI
 
 from .types.ctx import Ctx
 from .types.external import External
@@ -41,6 +44,7 @@ class Idx:
         self.storage = Storage(window)
         self.chat = Chat(window, self.storage)
         self.metadata = Metadata(window)
+        self.ui = UI(window)
 
         self.providers = {
             "json_file": JsonFileProvider(window),  # only for patching
@@ -98,7 +102,11 @@ class Idx:
         """
         self.storage.store(idx)
 
-    def remove_index(self, idx: str = "base", truncate: bool = False) -> bool:
+    def remove_index(
+            self,
+            idx: str = "base",
+            truncate: bool = False
+    ) -> bool:
         """
         Truncate index
 
@@ -125,10 +133,10 @@ class Idx:
     def index_files(
             self,
             idx: str = "base",
-            path: str = None,
-            replace: bool = None,
-            recursive: bool = None,
-    ) -> (dict, list):
+            path: Optional[str] = None,
+            replace: Optional[bool] = None,
+            recursive: Optional[bool] = None,
+    ) -> Tuple[Dict, List[str]]:
         """
         Index file or directory of files
 
@@ -138,10 +146,11 @@ class Idx:
         :param recursive: recursive indexing
         :return: dict with indexed files (path -> id), list with errors
         """
-        context = self.llm.get_service_context()
+        llm, embed_model = self.llm.get_service_context()
         index = self.storage.get(
             id=idx,
-            service_context=context,
+            llm=llm,
+            embed_model=embed_model,
         )  # get or create index
         files, errors = self.indexing.index_files(
             idx=idx,
@@ -165,7 +174,7 @@ class Idx:
             idx: str = "base",
             id: int = 0,
             from_ts: int = 0
-    ) -> (int, list):
+    ) -> Tuple[int, List[str]]:
         """
         Index records from db by meta id
 
@@ -174,10 +183,11 @@ class Idx:
         :param from_ts: timestamp from
         :return: num of indexed files, list with errors
         """
-        context = self.llm.get_service_context()
+        llm, embed_model = self.llm.get_service_context()
         index = self.storage.get(
             id=idx,
-            service_context=context,
+            llm=llm,
+            embed_model=embed_model,
         )  # get or create index
         num, errors = self.indexing.index_db_by_meta_id(
             idx=idx,
@@ -199,7 +209,7 @@ class Idx:
             self,
             idx: str = "base",
             from_ts: int = 0
-    ) -> (int, list):
+    ) -> Tuple[int, List[str]]:
         """
         Index records from db by meta id
 
@@ -207,10 +217,11 @@ class Idx:
         :param from_ts: timestamp from
         :return: num of indexed files, list with errors
         """
-        context = self.llm.get_service_context()
+        llm, embed_model = self.llm.get_service_context()
         index = self.storage.get(
             id=idx,
-            service_context=context,
+            llm=llm,
+            embed_model=embed_model,
         )  # get or create index
         num, errors = self.indexing.index_db_from_updated_ts(
             idx=idx,
@@ -230,10 +241,10 @@ class Idx:
     def index_urls(
             self,
             idx: str = "base",
-            urls: list = None,
+            urls: Optional[List[str]] = None,
             type: str = "webpage",
-            extra_args: dict = None
-    ) -> (dict, list):
+            extra_args: Optional[Dict[str, Any]] = None
+    ) -> Tuple[int, List[str]]:
         """
         Index URLs
 
@@ -243,10 +254,11 @@ class Idx:
         :param extra_args: extra args
         :return: num of indexed, list with errors
         """
-        context = self.llm.get_service_context()
+        llm, embed_model = self.llm.get_service_context()
         index = self.storage.get(
             id=idx,
-            service_context=context,
+            llm=llm,
+            embed_model=embed_model,
         )  # get or create index
         n, errors = self.indexing.index_urls(
             idx=idx,
@@ -269,10 +281,10 @@ class Idx:
             self,
             idx: str = "base",
             type: str = "webpage",
-            params: dict = None,
-            config: dict = None,
-            replace: bool = None,
-    ) -> (dict, list):
+            params: Optional[Dict[str, Any]] = None,
+            config: Optional[Dict[str, Any]] = None,
+            replace: Optional[bool] = None,
+    ) -> Tuple[int, list]:
         """
         Index URLs
 
@@ -286,10 +298,11 @@ class Idx:
         # update config params
         self.indexing.update_loader_args(type, config)
 
-        context = self.llm.get_service_context()
+        llm, embed_model = self.llm.get_service_context()
         index = self.storage.get(
             id=idx,
-            service_context=context,
+            llm=llm,
+            embed_model=embed_model,
         )  # get or create index
         n, errors = self.indexing.index_url(
             idx=idx,
@@ -310,7 +323,10 @@ class Idx:
             self.log("Error: " + str(errors))
         return n, errors
 
-    def get_idx_data(self, idx: str = None) -> dict:
+    def get_idx_data(
+            self,
+            idx: Optional[str] = None
+    ) -> Dict[str, Dict]:
         """
         Get indexed files data
 
@@ -329,7 +345,7 @@ class Idx:
                     indexes[idx] = self.items[store_id][idx].items
         return indexes
 
-    def get_by_idx(self, idx: int) -> str:
+    def get_by_idx(self, idx: int) -> Optional[str]:
         """
         Return idx by list index
 
@@ -341,7 +357,7 @@ class Idx:
             if idx < len(items):
                 return items[idx]['id']
 
-    def get_idx_by_name(self, name: str) -> int:
+    def get_idx_by_name(self, name: str) -> Optional[int]:
         """
         Return idx on list by name
 
@@ -354,7 +370,7 @@ class Idx:
                 if item['id'] == name:
                     return idx
 
-    def get_default_idx(self) -> str:
+    def get_default_idx(self) -> Optional[str]:
         """
         Return default idx
 
@@ -375,7 +391,7 @@ class Idx:
             return idx in self.items[store_id]
         return False
 
-    def get(self, idx: str) -> IndexItem:
+    def get(self, idx: str) -> Optional[IndexItem]:
         """
         Return index data from current storage
 
@@ -386,7 +402,7 @@ class Idx:
         if store_id in self.items and idx in self.items[store_id]:
             return self.items[store_id][idx]
 
-    def get_all(self) -> dict:
+    def get_all(self) -> Dict[str, IndexItem]:
         """
         Return all indexes in current store
 
@@ -397,12 +413,16 @@ class Idx:
             return self.items[store_id]
         return {}
 
-    def append(self, idx: str, files: dict):
+    def append(
+            self,
+            idx: str,
+            files: Dict[str, str]
+    ):
         """
         Append indexed files to index
 
         :param idx: index name
-        :param files: dict of indexed files
+        :param files: dict of indexed files (path -> doc_id)
         """
         # create store if not exists
         store_id = self.get_current_store()
@@ -445,7 +465,11 @@ class Idx:
                 self.items[store_id][idx].items[file_id]["id"] = doc_id
                 self.items[store_id][idx].items[file_id]["indexed_ts"] = ts
 
-    def remove_doc(self, idx: str, doc_id: str):
+    def remove_doc(
+            self,
+            idx: str,
+            doc_id: str
+    ):
         """
         Remove document from index
 
@@ -456,7 +480,11 @@ class Idx:
         if self.storage.remove_document(idx, doc_id):
             self.log("Removed document from index: " + idx + " - " + doc_id)
 
-    def remove_file(self, idx: str, file: str):
+    def remove_file(
+            self,
+            idx: str,
+            file: str
+    ):
         """
         Remove file from index
 
@@ -508,7 +536,7 @@ class Idx:
                     self.items[store_id][idx].id = idx
                     self.items[store_id][idx].name = idx
 
-    def get_idx_ids(self) -> list:
+    def get_idx_ids(self) -> List[Dict[str, str]]:
         """
         Get list of indexes
 
@@ -518,7 +546,10 @@ class Idx:
         data =  self.window.core.config.get('llama.idx.list')
         if data is not None:
             for item in data:
-                ids.append(item['id'])
+                name = item['name']
+                if name is None or name == "":
+                    name = item['id']
+                ids.append({item['id']: name})
         return ids
 
     def clear(self, idx: str):
@@ -532,7 +563,7 @@ class Idx:
             self.items[store_id][idx].items = {}
         self.get_provider().truncate(store_id, idx)
 
-    def get_counters(self, type: str) -> dict:
+    def get_counters(self, type: str) -> Dict[str, Dict[str, int]]:
         """
         Get counters (stats, count items by type [file, ctx, external])
 
