@@ -6,11 +6,15 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.05.01 03:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
+from typing import Optional, List
+
 from llama_index.core.llms import ChatMessage, MessageRole
 
+from pygpt_net.item.ctx import CtxItem
+
 
 class Context:
     def __init__(self, window=None):
@@ -25,7 +29,8 @@ class Context:
             self,
             input_prompt: str,
             system_prompt: str,
-            history: list = None
+            history: Optional[List[CtxItem]] = None,
+            multimodal: bool = False
     ):
         """
         Get messages from db
@@ -33,6 +38,7 @@ class Context:
         :param input_prompt: input prompt
         :param system_prompt: system prompt
         :param history: history
+        :param multimodal: multimodal flag
         :return: Messages
         """
         messages = []
@@ -60,16 +66,38 @@ class Context:
             )
             for item in items:
                 # input
-                if item.input is not None and item.input != "":
+                if item.final_input is not None and item.final_input != "":
                     messages.append(ChatMessage(
                         role=MessageRole.USER,
-                        content=item.input
+                        content=item.final_input
                     ))
                 # output
-                if item.output is not None and item.output != "":
+                if item.final_output is not None and item.final_output != "":
                     messages.append(ChatMessage(
                         role=MessageRole.ASSISTANT,
-                        content=item.output
+                        content=item.final_output
                     ))
 
         return messages
+
+    def add_user(self, query: str) -> ChatMessage:
+        """
+        Add user message
+
+        :param query: input query
+        """
+        return ChatMessage(
+            role=MessageRole.USER,
+            content=query,
+        )
+
+    def add_system(self, prompt: str) -> ChatMessage:
+        """
+        Add system message to db
+
+        :param prompt: system prompt
+        """
+        return ChatMessage(
+            role=MessageRole.SYSTEM,
+            content=prompt,
+        )
