@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.08.25 04:00:00                  #
+# Updated Date: 2025.01.16 01:00:00                  #
 # ================================================== #
 
 import datetime
@@ -14,12 +14,15 @@ import os.path
 import time
 
 from pathlib import Path
+from typing import Optional, Tuple, List, Dict, Any
+
 from sqlalchemy import text
 
 from llama_index.core.indices.base import BaseIndex
 from llama_index.core.schema import Document
 from llama_index.core import SimpleDirectoryReader
 
+from pygpt_net.item.model import ModelItem
 from pygpt_net.provider.loaders.base import BaseLoader
 from pygpt_net.utils import parse_args, pack_arg
 
@@ -86,19 +89,40 @@ class Indexing:
                             "key": key,
                             "value": loader.init_args[key],
                             "type": "str",  # default = str
+                            "label": key,
+                            "description": None,
                         }
                         # from config
                         if key in loader.args:
                             self.external_config[loader.id][key]["value"] = loader.args[key]
                         if key in loader.init_args_types:
                             self.external_config[loader.id][key]["type"] = loader.init_args_types[key]
+                        if key in loader.init_args_labels:
+                            self.external_config[loader.id][key]["label"] = loader.init_args_labels[key]
+                        if key in loader.init_args_desc:
+                            self.external_config[loader.id][key]["description"] = loader.init_args_desc[key]
 
             except ImportError as e:
                 msg = "Error while registering data loader: " + loader.id + " - " + str(e)
                 self.window.core.debug.log(msg)
                 self.window.core.debug.log(e)
 
-    def update_loader_args(self, loader: str, args: dict):
+    def get_loader(self, loader: str) -> Optional[BaseLoader]:
+        """
+        Get data loader by id
+
+        :param loader: loader id
+        :return: data loader instance
+        """
+        if loader in self.data_providers:
+            return self.data_providers[loader]
+        return None
+
+    def update_loader_args(
+            self,
+            loader: str,
+            args: Dict[str, Any]
+    ):
         """
         Update loader arguments
 
@@ -141,7 +165,7 @@ class Indexing:
             self.register_loader(loader)
         self.window.core.idx.log("Data loaders reloaded.")
 
-    def get_external_instructions(self) -> dict:
+    def get_external_instructions(self) -> Dict[str, Any]:
         """
         Get external instructions
 
@@ -149,7 +173,7 @@ class Indexing:
         """
         return self.external_instructions
 
-    def get_external_config(self) -> dict:
+    def get_external_config(self) -> Dict[str, Any]:
         """
         Get external config
 
@@ -176,7 +200,7 @@ class Indexing:
             if ext in extensions:
                 return loader["loader"]
 
-    def get_data_providers(self) -> dict:
+    def get_data_providers(self) -> Dict[str, BaseLoader]:
         """
         Get data providers
 
@@ -184,7 +208,11 @@ class Indexing:
         """
         return self.data_providers
 
-    def get_loader_arguments(self, id: str, type: str = "file") -> dict:
+    def get_loader_arguments(
+            self,
+            id: str,
+            type: str = "file"
+    ) -> Dict[str, Any]:
         """
         Get keyword arguments for loader
 
@@ -235,6 +263,8 @@ class Indexing:
             os.path.join(data_dir, ".interpreter.output.py"),
             os.path.join(data_dir, ".interpreter.input.py"),
             os.path.join(data_dir, ".interpreter.current.py"),
+            os.path.join(data_dir, ".interpreter.kernel.json"),
+            os.path.join(data_dir, ".canvas.html"),
         ]
         if path in excluded:
             return True
@@ -259,15 +289,25 @@ class Indexing:
             return False
         return True
 
-    def get_documents(self, path: str, force: bool = False) -> list[Document]:
+    def get_documents(
+            self,
+            path: str,
+            force: bool = False,
+            silent: bool = False,
+            loader_kwargs: Optional[Dict[str, Any]] = None,
+    ) -> List[Document]:
         """
-        Get documents from path
+        Get documents from path using data loaders
 
         :param path: path to data
         :param force: force reading
+        :param silent: disable logging
+        :param loader_kwargs: additional keyword arguments for loader
         :return: list of documents
         """
-        self.window.core.idx.log("Reading documents from path: {}".format(path))
+        # TODO: if .zip then unpack here, and return path to /tmp
+        if not silent:
+            self.window.core.idx.log("Reading documents from path: {}".format(path))
         if os.path.isdir(path):
             reader = SimpleDirectoryReader(
                 input_dir=path,
@@ -281,20 +321,35 @@ class Indexing:
 
             # check if not excluded extension
             if self.is_excluded(ext) and not force:
-                self.window.core.idx.log("Ignoring excluded extension: {}".format(ext))
+                if not silent:
+                    self.window.core.idx.log("Ignoring excluded extension: {}".format(ext))
                 return []
 
             # check if not excluded path
             if self.is_excluded_path(path) and not force:
-                self.window.core.idx.log("Ignoring excluded path: {}".format(path))
+                if not silent:
+                    self.window.core.idx.log("Ignoring excluded path: {}".format(path))
                 return []
 
+            # check if archive (zip, tar)
+            if self.window.core.filesystem.packer.is_archive(path):
+                tmp_path = self.window.core.filesystem.packer.unpack(path)
+                if tmp_path:
+                    return self.get_documents(tmp_path, force=force, silent=silent, loader_kwargs=loader_kwargs)
+
             if ext in self.loaders["file"]:
-                self.window.core.idx.log("Using loader for: {}".format(ext))
+                if not silent:
+                    self.window.core.idx.log("Using loader for: {}".format(ext))
                 reader = self.loaders["file"][ext]
-                documents = reader.load_data(file=Path(path))
+
+                # use custom loader method if available
+                if hasattr(reader, "load_data_custom") and loader_kwargs:
+                    documents = reader.load_data_custom(file=Path(path), **loader_kwargs)
+                else:
+                    documents = reader.load_data(file=Path(path))
             else:
-                self.window.core.idx.log("Using default SimpleDirectoryReader for: {}".format(ext))
+                if not silent:
+                    self.window.core.idx.log("Using default SimpleDirectoryReader for: {}".format(ext))
                 reader = SimpleDirectoryReader(input_files=[path])
                 documents = reader.load_data()
 
@@ -302,18 +357,87 @@ class Indexing:
         self.window.core.idx.metadata.append_file_metadata(documents, path)
         return documents
 
-    def read_text_content(self, path: str) -> str:
+    def read_text_content(
+            self,
+            path: str,
+            loader_kwargs: Optional[Dict[str, Any]] = None
+    ) -> Tuple[str, List[Document]]:
         """
         Get content from file using loaders
 
         :param path: path to file
-        :return: file content
+        :param loader_kwargs: additional keyword arguments for data loader
+        :return: text content, list of documents
         """
-        docs = self.get_documents(path, force=True)
+        docs = self.get_documents(
+            path,
+            force=True,  # allow excluded extensions
+            silent=True,
+            loader_kwargs=loader_kwargs,
+        )
         data = []
         for doc in docs:
             data.append(doc.text)
-        return "\n".join(data)
+        return "\n".join(data), docs
+
+    def read_web_content(
+            self,
+            url: str,
+            type: str = "webpage",
+            extra_args: Optional[Dict[str, Any]] = None
+    ) -> Tuple[str, List[Document]]:
+        """
+        Get content from external resource
+
+        :param url: external url to index
+        :param type: type of URL (webpage, feed, etc.)
+        :param extra_args: extra arguments for loader
+        :return: text content, list of documents
+        """
+        docs = self.read_web(url, type, extra_args)
+        data = []
+        for doc in docs:
+            data.append(doc.text)
+        return "\n".join(data), docs
+
+    def read_web(
+            self,
+            url: str,
+            type: str = "webpage",
+            extra_args: Optional[Dict[str, Any]] = None,
+    ) -> List[Document]:
+        """
+        Read data from external resource
+
+        :param url: external url to index
+        :param type: type of URL (webpage, feed, etc.)
+        :param extra_args: extra arguments for loader
+        :return: list of documents
+        """
+        documents = []
+
+        # check if web loader for defined type exists
+        if type not in self.loaders["web"]:
+            raise ValueError("No web loader for type: {}".format(type))
+
+        try:
+            if "url" not in extra_args:
+                extra_args["url"] = url
+
+            # get unique external content identifier
+            unique_id = self.data_providers[type].get_external_id(extra_args)
+            self.window.core.idx.log("Loading web documents from: {}".format(unique_id))
+            self.window.core.idx.log("Using web loader for type: {}".format(type))
+
+            args = self.data_providers[type].prepare_args(**extra_args)
+
+            # get documents from external resource
+            documents = self.loaders["web"][type].load_data(
+                **args
+            )
+        except Exception as e:
+            self.window.core.debug.log(e)
+        return documents
 
     def prepare_document(self, doc: Document):
         """
@@ -322,19 +446,19 @@ class Indexing:
         :param doc: Document
         """
         # fix empty date in Pinecode
-        if "last_accessed_date" in doc.extra_info and doc.extra_info["last_accessed_date"] is None:
-            if "creation_date" in doc.extra_info:
-                doc.extra_info["last_accessed_date"] = doc.extra_info["creation_date"]
+        if "last_accessed_date" in doc.metadata and doc.metadata["last_accessed_date"] is None:
+            if "creation_date" in doc.metadata:
+                doc.metadata["last_accessed_date"] = doc.metadata["creation_date"]
 
     def index_files(
             self,
             idx: str,
             index: BaseIndex,
-            path: str = None,
+            path: Optional[str] = None,
             is_tmp: bool = False,
-            replace: bool = None,
-            recursive: bool = None
-    ) -> tuple:
+            replace: Optional[bool] = None,
+            recursive: Optional[bool] = None
+    ) -> Tuple[dict, list]:
         """
         Index all files in directory
 
@@ -364,19 +488,18 @@ class Indexing:
 
         for file in files:   # per file to allow use of multiple loaders
             try:
-                # remove old file from index if exists
-                file_id = self.window.core.idx.files.get_id(file)
-
                 if self.is_stopped():  # force stop
                     break
 
                 # force replace or not old document
                 if replace is not None:
                     if replace:
+                        file_id = self.window.core.idx.files.get_id(file)
                         self.remove_old_file(idx, file_id, force=True)
                 else:
                     # if auto, only replace if not temporary
                     if not is_tmp:
+                        file_id = self.window.core.idx.files.get_id(file)
                         self.remove_old_file(idx, file_id)
 
                 # index new version of file
@@ -402,10 +525,10 @@ class Indexing:
             self,
             idx: str,
             index: BaseIndex,
-            path: str = None,
+            path: Optional[str] = None,
             is_tmp: bool = False,
-            replace: bool = None
-    ) -> tuple:
+            replace: Optional[bool] = None
+    ) -> Tuple[dict, list]:
         """
         Index all files in directory and subdirectories recursively.
 
@@ -494,7 +617,10 @@ class Indexing:
 
         return indexed, errors
 
-    def get_db_data_from_ts(self, updated_ts: int = 0) -> list:
+    def get_db_data_from_ts(
+            self,
+            updated_ts: int = 0
+    ) -> List[Document]:
         """
         Get data from database from timestamp
 
@@ -534,7 +660,10 @@ class Indexing:
                 documents.append(doc)
         return documents
 
-    def get_db_meta_ids_from_ts(self, updated_ts: int = 0) -> list:
+    def get_db_meta_ids_from_ts(
+            self,
+            updated_ts: int = 0
+    ) -> List[int]:
         """
         Get IDs of meta from database from timestamp
 
@@ -558,7 +687,11 @@ class Indexing:
                 ids.append(data["id"])
         return ids
 
-    def get_db_data_by_id(self, id: int = 0, updated_ts: int = 0) -> list:
+    def get_db_data_by_id(
+            self,
+            id: int = 0,
+            updated_ts: int = 0
+    ) -> List[Document]:
         """
         Get data from database by meta id
 
@@ -596,7 +729,13 @@ class Indexing:
                 documents.append(doc)
         return documents
 
-    def index_db_by_meta_id(self, idx: str, index: BaseIndex, id: int = 0, from_ts: int = 0) -> (int, list):
+    def index_db_by_meta_id(
+            self,
+            idx: str,
+            index: BaseIndex,
+            id: int = 0,
+            from_ts: int = 0
+    ) -> Tuple[int, List[str]]:
         """
         Index data from database by meta id
 
@@ -614,7 +753,8 @@ class Indexing:
                 self.window.core.idx.log("Indexing documents from database by meta id: {}".format(id))
                 self.remove_old_meta_id(idx, id)
             elif from_ts > 0:
-                self.window.core.idx.log("Indexing documents from database by meta id: {} from timestamp: {}".format(id, from_ts))
+                self.window.core.idx.log("Indexing documents from database by meta id: {} from timestamp: {}".
+                                         format(id, from_ts))
 
             # get items from database
             documents = self.get_db_data_by_id(id, from_ts)
@@ -624,7 +764,8 @@ class Indexing:
 
                 self.index_document(index, d)
                 doc_id = d.id_
-                self.window.core.idx.log("Inserted ctx DB document: {} / {}, id: {}, metadata: {}".format(n+1, len(documents), d.id_, d.metadata))
+                self.window.core.idx.log("Inserted ctx DB document: {} / {}, id: {}, metadata: {}".
+                                         format(n+1, len(documents), d.id_, d.metadata))
                 self.window.core.ctx.idx.set_meta_as_indexed(id, idx, doc_id)  # update ctx
                 n += 1
         except Exception as e:
@@ -632,7 +773,12 @@ class Indexing:
             self.window.core.debug.log(e)
         return n, errors
 
-    def index_db_from_updated_ts(self, idx: str, index: BaseIndex, from_ts: int = 0) -> (int, list):
+    def index_db_from_updated_ts(
+            self,
+            idx: str,
+            index: BaseIndex,
+            from_ts: int = 0
+    ) -> Tuple[int, List[str]]:
         """
         Index data from database from timestamp
 
@@ -660,10 +806,10 @@ class Indexing:
             index: BaseIndex,
             url: str,
             type="webpage",
-            extra_args: dict = None,
+            extra_args: Optional[Dict[str, Any]] = None,
             is_tmp: bool = False,
-            replace: bool = None
-    ) -> (int, list):
+            replace: Optional[bool] = None
+    ) -> Tuple[int, List[str]]:
         """
         Index data from external (remote) resource
 
@@ -733,7 +879,8 @@ class Indexing:
                         idx=idx,
                         doc_id=doc_id,
                     )  # update external index
-                self.window.core.idx.log("Inserted web document: {} / {}, id: {}, metadata: {}".format(n+1, len(documents), d.id_, d.metadata))
+                self.window.core.idx.log("Inserted web document: {} / {}, id: {}, metadata: {}".
+                                         format(n+1, len(documents), d.id_, d.metadata))
                 n += 1
         except Exception as e:
             errors.append(str(e))
@@ -746,9 +893,9 @@ class Indexing:
             index: BaseIndex,
             urls: list,
             type="webpage",
-            extra_args: dict = None,
+            extra_args: Optional[Dict[str, Any]] = None,
             is_tmp: bool = False
-    ) -> (int, list):
+    ) -> Tuple[int, List[str]]:
         """
         Index data from URLs
 
@@ -786,7 +933,12 @@ class Indexing:
             errors.extend(errs)
         return n, errors
 
-    def remove_old_meta_id(self, idx: str, id: int = 0, force: bool = False) -> bool:
+    def remove_old_meta_id(
+            self,
+            idx: str,
+            id: int = 0,
+            force: bool = False
+    ) -> bool:
         """
         Remove old meta id from index
 
@@ -814,7 +966,12 @@ class Indexing:
                 return True
         return False
 
-    def remove_old_file(self, idx: str, file_id: str, force: bool = False):
+    def remove_old_file(
+            self,
+            idx: str,
+            file_id: str,
+            force: bool = False
+    ) -> bool:
         """
         Remove old file from index
 
@@ -842,7 +999,13 @@ class Indexing:
                 return True
         return False
 
-    def remove_old_external(self, idx: str, content: str, type: str, force: bool = False):
+    def remove_old_external(
+            self,
+            idx: str,
+            content: str,
+            type: str,
+            force: bool = False
+    ) -> bool:
         """
         Remove old file from index
 
@@ -871,7 +1034,11 @@ class Indexing:
                 return True
         return False
 
-    def index_document(self, index: BaseIndex, doc: Document):
+    def index_document(
+            self,
+            index: BaseIndex,
+            doc: Document
+    ):
         """
         Index document
 
@@ -879,13 +1046,137 @@ class Indexing:
         :param doc: document
         """
         self.apply_rate_limit()  # apply RPM limit
+        """
         try:
             # display embedding model info
-            self.window.core.idx.log("Embedding model: {}".format(index.service_context.embed_model.model_name))
+            if index._embed_model is not None:
+                self.window.core.idx.log("Embedding model: {}".format(index.service_context.embed_model.model_name))
         except Exception as e:
             self.window.core.debug.log(e)
+        """
         index.insert(document=doc)
 
+    def index_attachment(
+            self,
+            file_path: str,
+            index_path: str,
+            model: Optional[ModelItem] = None,
+            documents: Optional[List[Document]] = None,
+    ) -> list:
+        """
+        Index context attachment
+
+        :param file_path: path to file to index
+        :param index_path: index path
+        :param model: model
+        :param documents: list of documents (optional)
+        :return: response
+        """
+        if model is None:
+            model = self.window.core.models.from_defaults()
+
+        llm, embed_model = self.window.core.idx.llm.get_service_context(model=model)
+        index = self.window.core.idx.storage.get_ctx_idx(
+            index_path,
+            llm=llm,
+            embed_model=embed_model,
+        )  # get or create ctx index
+
+        idx = "tmp:{}".format(index_path)  # tmp index id
+        self.window.core.idx.log("Indexing to context attachment index: {}...".format(idx))
+
+        doc_ids = []
+        if documents is None:
+            documents = self.get_documents(file_path)
+        for d in documents:
+            if self.is_stopped():  # force stop
+                break
+            self.prepare_document(d)
+            self.index_document(index, d)
+            doc_ids.append(d.id_)  # add to index
+
+        self.window.core.idx.storage.store_ctx_idx(index_path, index)
+        return doc_ids
+
+    def index_attachment_web(
+            self,
+            url: str,
+            index_path: str,
+            model: Optional[ModelItem] = None,
+            documents: Optional[List[Document]] = None,
+    ) -> list:
+        """
+        Index context attachment
+
+        :param url: URL to index
+        :param index_path: index path
+        :param model: model
+        :param documents: list of documents (optional)
+        :return: response
+        """
+        if model is None:
+            model = self.window.core.models.from_defaults()
+
+        llm, embed_model = self.window.core.idx.llm.get_service_context(model=model)
+        index = self.window.core.idx.storage.get_ctx_idx(index_path, llm, embed_model)  # get or create ctx index
+
+        idx = "tmp:{}".format(index_path)  # tmp index id
+        self.window.core.idx.log("Indexing to context attachment index: {}...".format(idx))
+
+        web_type = self.get_webtype(url)
+        doc_ids = []
+        if documents is None:
+            documents = self.read_web(
+                url=url,
+                type=web_type,
+                extra_args={},
+            )
+        for d in documents:
+            if self.is_stopped():  # force stop
+                break
+            self.prepare_document(d)
+            self.index_document(index, d)
+            doc_ids.append(d.id_)  # add to index
+
+        self.window.core.idx.storage.store_ctx_idx(index_path, index)
+        return doc_ids
+
+    def get_webtype(self, url: str) -> str:
+        """
+        Get web loader type by URL
+
+        :param url: URL
+        :return: web loader type
+        """
+        type = "webpage"  # default
+        for id in self.data_providers:
+            loader = self.data_providers[id]
+            if hasattr(loader, "is_supported_attachment"):
+                if loader.is_supported_attachment(url):
+                    type = id
+                    break
+        print("Selected web data loader: {}".format(type))
+        return type
+
+    def remove_attachment(
+            self,
+            index_path: str,
+            doc_id: str
+    ) -> bool:
+        """
+        Remove document from index
+
+        :param index_path: index path
+        :param doc_id: document ID
+        :return: True if success
+        """
+        model = self.window.core.models.from_defaults()
+        llm, embed_model = self.window.core.idx.llm.get_service_context(model=model)
+        index = self.window.core.idx.storage.get_ctx_idx(index_path, llm, embed_model)  # get or create ctx index
+        index.delete_ref_doc(doc_id)
+        self.window.core.idx.storage.store_ctx_idx(index_path, index)
+        return True
+
     def apply_rate_limit(self):
         """Apply API calls RPM limit"""
         max_per_minute = 60
