@@ -6,11 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.08 18:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
 import os
+from typing import Optional, List, Dict
 
+from pygpt_net.core.events import Event
 from pygpt_net.item.ctx import CtxItem
 from pygpt_net.utils import trans
 
@@ -20,7 +22,6 @@ import pygpt_net.js_rc
 import pygpt_net.css_rc
 import pygpt_net.fonts_rc
 
-
 class Body:
 
     def __init__(self, window=None):
@@ -46,23 +47,40 @@ class Body:
 
         :return: CSS styles
         """
-        dark_styles = ["dracula", "fruity", "github-dark", "gruvbox-dark", "inkpot", "material", "monokai",
-                       "native", "nord", "nord-darker", "one-dark", "paraiso-dark", "rrt", "solarized-dark",
-                       "stata-dark", "vim", "zenburn"]
-        style = self.window.core.config.get("render.code_syntax")
-        if style is None or style == "":
-            style = "default"
+        syntax_dark = [
+            "dracula",
+            "fruity",
+            "github-dark",
+            "gruvbox-dark",
+            "inkpot",
+            "material",
+            "monokai",
+            "native",
+            "nord",
+            "nord-darker",
+            "one-dark",
+            "paraiso-dark",
+            "rrt",
+            "solarized-dark",
+            "stata-dark",
+            "vim",
+            "zenburn",
+        ]
+        syntax_style = self.window.core.config.get("render.code_syntax")
+        if syntax_style is None or syntax_style == "":
+            syntax_style = "default"
         fonts_path = os.path.join(self.window.core.config.get_app_path(), "data", "fonts").replace("\\", "/")
         stylesheet = self.window.controller.theme.markdown.get_web_css().replace('%fonts%', fonts_path)
-        if style in dark_styles:
+        if syntax_style in syntax_dark:
             stylesheet += "pre { color: #fff; }"
         else:
             stylesheet += "pre { color: #000; }"
-        content = stylesheet + """
-        """ + self.highlight.get_style_defs()
-        return content
+        return stylesheet + " " + self.highlight.get_style_defs()
 
-    def prepare_action_icons(self, ctx: CtxItem) -> str:
+    def prepare_action_icons(
+            self,
+            ctx: CtxItem
+    ) -> str:
         """
         Append action icons
 
@@ -70,14 +88,19 @@ class Body:
         :return: HTML code
         """
         html = ""
-        show_edit = self.window.core.config.get('ctx.edit_icons')
+        show_edit = True
+        # show_edit = self.window.core.config.get('ctx.edit_icons')
         icons_html = "".join(self.get_action_icons(ctx, all=show_edit))
         if icons_html != "":
             extra = "<div class=\"action-icons\" data-id=\"{}\">{}</div>".format(ctx.id, icons_html)
             html += extra
         return html
 
-    def get_action_icons(self, ctx: CtxItem, all: bool = False) -> list:
+    def get_action_icons(
+            self,
+            ctx: CtxItem,
+            all: bool = False
+    ) -> List[str]:
         """
         Get action icons for context item
 
@@ -140,7 +163,12 @@ class Body:
                 )
         return icons
 
-    def get_icon(self, icon: str, title: str = None, item: CtxItem = None) -> str:
+    def get_icon(
+            self,
+            icon: str,
+            title: Optional[str] = None,
+            item: Optional[CtxItem] = None
+    ) -> str:
         """
         Get icon
 
@@ -153,7 +181,12 @@ class Body:
         return '<img src="file://{}" class="action-img" title="{}" alt="{}" data-id="{}">'.format(
             icon, title, title, item.id)
 
-    def get_image_html(self, url: str, num: int = None, num_all: int = None) -> str:
+    def get_image_html(
+            self,
+            url: str,
+            num: Optional[int] = None,
+            num_all: Optional[int] = None
+    ) -> str:
         """
         Get image HTML
 
@@ -166,14 +199,21 @@ class Body:
         if num is not None and num_all is not None and num_all > 1:
             num_str = " [{}]".format(num)
         url, path = self.window.core.filesystem.extract_local_url(url)
-        return """<a href="{url}"><img src="{path}" class="image"></a>
-        <p><b>{prefix}{num}:</b> <a href="{url}">{path}</a></p>""". \
+        basename = os.path.basename(path)
+        return """<div class="extra-src-img-box" title="{url}"><div class="img-outer"><div class="img-wrapper"><a href="{url}"><img src="{path}" class="image"></a></div>
+        <a href="{url}" class="title">{title}</a></div></div>""". \
             format(prefix=trans('chat.prefix.img'),
                    url=url,
+                   title=basename,
                    path=path,
                    num=num_str)
 
-    def get_url_html(self, url: str, num: int = None, num_all: int = None) -> str:
+    def get_url_html(
+            self,
+            url: str,
+            num: Optional[int] = None,
+            num_all: Optional[int] = None
+    ) -> str:
         """
         Get URL HTML
 
@@ -182,15 +222,21 @@ class Body:
         :param num_all: number of all URLs
         :return: HTML code
         """
+        icon_path = os.path.join(
+            self.window.core.config.get_app_path(),
+            "data", "icons", "language.svg"
+        )
+        icon = '<img src="file://{}" class="extra-src-icon">'.format(icon_path)
         num_str = ""
         if num is not None and num_all is not None and num_all > 1:
             num_str = " [{}]".format(num)
-        return """<b>{prefix}{num}:</b> <a href="{url}">{url}</a>""". \
-            format(prefix=trans('chat.prefix.url'),
-                   url=url,
-                   num=num_str)
+        return """{icon}<a href="{url}" title="{url}">{url}</a> <small>{num}</small>""". \
+            format(url=url,
+                   num=num_str,
+                   icon=icon,
+            )
 
-    def get_docs_html(self, docs: list) -> str:
+    def get_docs_html(self, docs: List[Dict]) -> str:
         """
         Get Llama-index doc metadata HTML
 
@@ -222,14 +268,27 @@ class Body:
         except Exception as e:
             pass
 
+        icon_path = os.path.join(
+            self.window.core.config.get_app_path(),
+            "data", "icons", "db.svg"
+        )
+        icon = '<img src="file://{}" class="extra-src-icon">'.format(icon_path)
         if html_sources != "":
-            html += "<p><small><b>{prefix}:</b></small></p>".format(prefix=trans('chat.prefix.doc'))
+            html += "<p>{icon}<small><b>{prefix}:</b></small></p>".format(
+                prefix=trans('chat.prefix.doc'),
+                icon=icon,
+            )
             html += "<div class=\"cmd\">"
             html += "<p>" + html_sources + "</p>"
             html += "</div> "
         return html
 
-    def get_file_html(self, url: str, num: int = None, num_all: int = None) -> str:
+    def get_file_html(
+            self,
+            url: str,
+            num: Optional[int] = None,
+            num_all: Optional[int] = None
+    ) -> str:
         """
         Get file HTML
 
@@ -238,17 +297,63 @@ class Body:
         :param num_all: number of all files
         :return: HTML code
         """
+        icon_path = os.path.join(
+            self.window.core.config.get_app_path(),
+            "data", "icons", "attachments.svg"
+        )
+        icon = '<img src="file://{}" class="extra-src-icon">'.format(icon_path)
         num_str = ""
         if num is not None and num_all is not None and num_all > 1:
             num_str = " [{}]".format(num)
         url, path = self.window.core.filesystem.extract_local_url(url)
-        return """<div><b>{prefix}{num}:</b> <a href="{url}">{path}</a></div>""". \
-            format(prefix=trans('chat.prefix.file'),
-                   url=url,
+        return """{icon} <b>{num}</b> <a href="{url}">{path}</a>""". \
+            format(url=url,
                    path=path,
-                   num=num_str)
+                   num=num_str,
+                   icon=icon,
+            )
 
-    def get_html(self) -> str:
+    def prepare_tool_extra(self, ctx: CtxItem) -> str:
+        """
+        Prepare footer extra
+
+        :param ctx: context item
+        :return: HTML code
+        """
+        html = ""
+        if ctx.extra is not None and ctx.extra != "":
+            html += "<div class=\"msg-extra\">"
+
+            # single tool
+            if "plugin" in ctx.extra:
+                event = Event(Event.TOOL_OUTPUT_RENDER, {
+                    'tool': ctx.extra["plugin"],
+                    'html': '',
+                    'multiple': False,
+                    'content': ctx.extra,  # tool output
+                })
+                event.ctx = ctx
+                self.window.dispatch(event, all=True)  # handle by plugins
+                html += "<div class=\"tool-output-block\">" + event.data['html'] + "</div>"
+
+            # multiple tools, list
+            elif "tool_output" in ctx.extra and isinstance(ctx.extra["tool_output"], list):
+                for key, tool in enumerate(ctx.extra["tool_output"]):
+                    if "plugin" not in tool:
+                        continue
+                    event = Event(Event.TOOL_OUTPUT_RENDER, {
+                        'tool': tool["plugin"],
+                        'html': '',
+                        'multiple': True,
+                        'content': tool,  # tool output[]
+                    })
+                    event.ctx = ctx
+                    self.window.dispatch(event, all=True)
+                    html += "<div class=\"tool-output-block\">" + event.data['html'] + "</div>"
+            html += "</div>"
+        return html
+
+    def get_html(self, pid: int) -> str:
         """
         Build webview HTML code
 
@@ -256,12 +361,14 @@ class Body:
         """
         classes = []
         classes_str = ""
+        style = self.window.core.config.get("theme.style", "blocks")
         if self.window.core.config.get('render.blocks'):
             classes.append("display-blocks")
         if self.window.core.config.get('ctx.edit_icons'):
             classes.append("display-edit-icons")
         if self.is_timestamp_enabled():
             classes.append("display-timestamp")
+        classes.append("theme-" + style)
         if classes:
             classes_str = ' class="' + " ".join(classes) + '"'
 
@@ -291,13 +398,14 @@ class Body:
         let scrollTimeout = null;
         let prevScroll = 0;
         let bridge;
+        let pid = """ + str(pid) + """
         new QWebChannel(qt.webChannelTransport, function (channel) {
             bridge = channel.objects.bridge;
         });
         history.scrollRestoration = "manual";
         document.addEventListener('keydown', function(event) {
             if (event.ctrlKey && event.key === 'f') {
-                window.location.href = 'bridge://open_find'; // send to bridge
+                window.location.href = 'bridge://open_find:' + pid; // send to bridge
                 event.preventDefault();
             }
             if (event.key === 'Escape') {
@@ -305,6 +413,11 @@ class Body:
                 event.preventDefault();
             }
         });
+        document.addEventListener('click', function(event) {
+            if (event.target.tagName !== 'A' && !event.target.closest('a')) {
+                window.location.href = 'bridge://focus';
+            }
+        });
         function highlightCode() {
             document.querySelectorAll('pre code').forEach(el => {
                 if (!el.classList.contains('hljs')) hljs.highlightElement(el);
