@@ -6,10 +6,11 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.08.27 05:00:00                  #
+# Updated Date: 2024.12.12 01:00:00                  #
 # ================================================== #
 
 import os.path
+import re
 import subprocess
 import docker
 
@@ -26,6 +27,14 @@ class Runner:
         self.plugin = plugin
         self.signals = None
 
+    def attach_signals(self, signals):
+        """
+        Attach signals
+
+        :param signals: signals
+        """
+        self.signals = signals
+
     def send_interpreter_input(self, data: str):
         """
         Send input to subprocess
@@ -45,6 +54,15 @@ class Runner:
         if self.signals is not None:
             self.signals.output.emit(data, type)
 
+    def send_html_output(self, data: str):
+        """
+        Send HTML output to canvas
+
+        :param data: HTML code
+        """
+        if self.signals is not None:
+            self.signals.html_output.emit(data)
+
     def handle_result(self, stdout, stderr):
         """
         Handle result from subprocess
@@ -84,6 +102,60 @@ class Runner:
         )
         return result
 
+    def extract_data_paths(self, text):
+        """
+        Extracts all file paths starting with '/data' from the given text.
+
+        Parameters:
+        text (str): The input text from which to extract paths.
+
+        Returns:
+        list: A list of paths starting with '/data'.
+        """
+        if text is None:
+            return []
+        pattern = r"(/data/[^\s'\";]+)"
+        paths = re.findall(pattern, text)
+        return paths
+
+    def extract_files(self, ctx: CtxItem, response: str) -> list:
+        """
+        Extract files from response
+
+        :param ctx: CtxItem
+        :param response: response
+        :return: files list
+        """
+        images_list = []
+        local_path = os.path.join(self.plugin.window.core.config.get_user_dir('data'), "ipython")
+        paths = self.extract_data_paths(response)
+        dir_suffix = "/"
+        if self.plugin.window.core.platforms.is_windows():
+            dir_suffix = "\\"
+        for i, file in enumerate(paths):
+            paths[i] = file.replace("/data/", local_path + dir_suffix)
+        for path in paths:
+            if path.strip().split(".")[-1].lower() in ["png", "jpg", "jpeg", "gif", "bmp", "tiff"]:
+                images_list.append(path)
+
+        # append to ctx
+        ctx.files = paths
+        ctx.images = self.plugin.window.core.filesystem.make_local_list(list(images_list))
+        return paths
+
+    def handle_result_ipython(self, ctx: CtxItem, response) -> str:
+        """
+        Handle result from ipython container, check for files and images
+
+        :param ctx: CtxItem
+        :param response: response
+        :return: result
+        """
+        paths = self.extract_files(ctx, response)
+        if len(paths) == 0:
+            self.extract_files(ctx, ctx.input)
+        return response
+
     def is_sandbox(self) -> bool:
         """
         Check if sandbox is enabled
@@ -92,6 +164,14 @@ class Runner:
         """
         return self.plugin.get_option_value('sandbox_docker')
 
+    def is_sandbox_ipython(self) -> bool:
+        """
+        Check if sandbox is enabled for IPython
+
+        :return: True if sandbox is enabled
+        """
+        return self.plugin.get_option_value('sandbox_ipython')
+
     def get_docker(self) -> docker.client.DockerClient:
         """
         Get docker client
@@ -129,18 +209,8 @@ class Runner:
         :param cmd: command to run
         :return: response
         """
-        client = self.get_docker()
-        mapping = self.get_volumes()
-        response = None
         try:
-            response = client.containers.run(
-                self.get_docker_image(),
-                cmd,
-                volumes=mapping,
-                working_dir="/data",
-                stdout=True,
-                stderr=True,
-            )
+            response = self.plugin.docker.execute(cmd)
         except Exception as e:
             # self.error(e)
             response = str(e).encode("utf-8")
@@ -320,58 +390,131 @@ class Runner:
             "context": "PYTHON OUTPUT:\n--------------------------------\n" + self.parse_result(result),
         }
 
-    def sys_exec_host(self, ctx: CtxItem, item: dict, request: dict) -> dict:
+    def ipython_execute_new(self, ctx, item: dict, request: dict, all: bool = False) -> dict:
         """
-        Execute system command on host
+        Execute code in IPython interpreter (new kernel)
 
         :param ctx: CtxItem
         :param item: command item
         :param request: request item
+        :param all: execute all
         :return: response dict
         """
-        msg = "Executing system command: {}".format(item["params"]['command'])
-        self.log(msg)
-        self.log("Running command: {}".format(item["params"]['command']))
+        sandbox = self.is_sandbox_ipython()
+        data = item["params"]['code']
+        if not all:
+            path = self.plugin.window.tools.get("interpreter").file_current
+            if "path" in item["params"]:
+                path = item["params"]['path']
+            msg = "Saving Python file: {}".format(path)
+            self.log(msg, sandbox=sandbox)
+            with open(self.prepare_path(path, on_host=True), 'w', encoding="utf-8") as file:
+                file.write(data)
+        else:
+            path = self.plugin.window.tools.get("interpreter").file_input
+
+        with open(self.prepare_path(path, on_host=True), 'r', encoding="utf-8") as file:
+            data = file.read()
+
+        self.append_input(data)
+        self.send_interpreter_input(data)  # send input to interpreter tool
+
+        # run code in IPython interpreter
+        msg = "Executing Python code: {}".format(item["params"]['code'])
+        self.log(msg, sandbox=sandbox)
+        self.log("Connecting to IPython interpreter...", sandbox=sandbox)
         try:
-            process = subprocess.Popen(
-                item["params"]['command'],
-                shell=True,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
-            )
-            stdout, stderr = process.communicate()
+            self.log("Please wait...", sandbox=sandbox)
+            result = self.plugin.get_interpreter().execute(data, current=False)
+            result = self.handle_result_ipython(ctx, result)
+            self.log("Python Code Executed.", sandbox=sandbox)
         except Exception as e:
             self.error(e)
-            stdout = None
-            stderr = str(e).encode("utf-8")
-        result = self.handle_result(stdout, stderr)
+            result = str(e)
         return {
             "request": request,
             "result": str(result),
-            "context": "SYS OUTPUT:\n--------------------------------\n" + self.parse_result(result),
+            "context": "IPYTHON OUTPUT:\n--------------------------------\n" + self.parse_result(result),
         }
 
-    def sys_exec_sandbox(self, ctx: CtxItem, item: dict, request: dict) -> dict:
+    def ipython_execute(self, ctx, item: dict, request: dict, all: bool = False) -> dict:
         """
-        Execute system command in sandbox (docker)
+        Execute code in IPython interpreter (current kernel)
 
         :param ctx: CtxItem
         :param item: command item
         :param request: request item
+        :param all: execute all
         :return: response dict
         """
-        msg = "Executing system command: {}".format(item["params"]['command'])
-        self.log(msg, sandbox=True)
-        self.log(
-            "Running command: {}".format(item["params"]['command']),
-            sandbox=True,
-        )
-        response = self.run_docker(item["params"]['command'])
-        result = self.handle_result_docker(response)
+        sandbox = self.is_sandbox_ipython()
+        data = item["params"]['code']
+        if not all:
+            path = self.plugin.window.tools.get("interpreter").file_current
+            if "path" in item["params"]:
+                path = item["params"]['path']
+            msg = "Saving Python file: {}".format(path)
+            self.log(msg, sandbox=sandbox)
+            with open(self.prepare_path(path, on_host=True), 'w', encoding="utf-8") as file:
+                file.write(data)
+        else:
+            path = self.plugin.window.tools.get("interpreter").file_input
+
+        with open(self.prepare_path(path, on_host=True), 'r', encoding="utf-8") as file:
+            data = file.read()
+
+        self.append_input(data)
+        self.send_interpreter_input(data)  # send input to interpreter tool
+
+        # run code in IPython interpreter
+        msg = "Executing Python code: {}".format(item["params"]['code'])
+        self.log(msg, sandbox=sandbox)
+        self.log("Connecting to IPython interpreter...", sandbox=sandbox)
+        try:
+            self.log("Please wait...", sandbox=sandbox)
+            result = self.plugin.get_interpreter().execute(data, current=True)
+            result = self.handle_result_ipython(ctx, result)
+            self.log("Python Code Executed.", sandbox=sandbox)
+        except Exception as e:
+            self.error(e)
+            result = str(e)
         return {
             "request": request,
             "result": str(result),
-            "context": "SYS OUTPUT:\n--------------------------------\n" + self.parse_result(result),
+            "context": "IPYTHON OUTPUT:\n--------------------------------\n" + self.parse_result(result),
+        }
+
+    def ipython_kernel_restart(self, ctx, item: dict, request: dict, all: bool = False) -> dict:
+        """
+        Execute code in IPython interpreter (current kernel)
+
+        :param ctx: CtxItem
+        :param item: command item
+        :param request: request item
+        :param all: execute all
+        :return: response dict
+        """
+        sandbox = self.is_sandbox_ipython()
+        self.append_input("")
+        self.send_interpreter_input("")  # send input to interpreter tool
+
+        # restart IPython interpreter
+        self.log("Connecting to IPython interpreter...", sandbox=sandbox)
+        try:
+            self.log("Restarting IPython kernel...", sandbox=sandbox)
+            response = self.plugin.get_interpreter().restart_kernel()
+        except Exception as e:
+            self.error(e)
+            response = False
+        if response:
+            result = "Kernel restarted"
+        else:
+            result = "Kernel not restarted"
+        self.log(result, sandbox=sandbox)
+        return {
+            "request": request,
+            "result": str(result),
+            "context": "IPYTHON OUTPUT:\n--------------------------------\n" + self.parse_result(result),
         }
 
     def parse_result(self, result):
