@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.08 19:00:00                  #
+# Updated Date: 2025.01.31 19:00:00                  #
 # ================================================== #
 
 import copy
@@ -1565,11 +1565,277 @@ class Patch:
                     data["tabs.data"] = self.window.core.tabs.from_defaults()
                 updated = True
 
-            # < 2.4.3
-            if old < parse_version("2.4.3"):
-                print("Migrating config from < 2.4.3...")
-                # css upgrade
-                self.window.core.updater.patch_css('web.css', True)  # force update
+            # < 2.4.7
+            if old < parse_version("2.4.7"):
+                print("Migrating config from < 2.4.7...")
+                self.window.core.plugins.reset_options("cmd_mouse_control", [
+                    "prompt",
+                ])
+                updated = True
+
+            # < 2.4.10
+            if old < parse_version("2.4.10"):
+                print("Migrating config from < 2.4.10...")
+                if 'prompt.agent.continue.llama' not in data:
+                    data["prompt.agent.continue.llama"] = self.window.core.config.get_base('prompt.agent.continue.llama')
+                if 'agent.llama.idx' not in data:
+                    data["agent.llama.idx"] = self.window.core.config.get_base('agent.llama.idx')
+                if 'agent.llama.steps' not in data:
+                    data["agent.llama.steps"] = self.window.core.config.get_base('agent.llama.steps')
+                if 'agent.llama.provider' not in data:
+                    data["agent.llama.provider"] = self.window.core.config.get_base('agent.llama.provider')
+                if 'agent.llama.verbose' not in data:
+                    data["agent.llama.verbose"] = self.window.core.config.get_base('agent.llama.verbose')
+                data["agent.goal.notify"] = False  # disable by default
+                updated = True
+
+            # < 2.4.11
+            if old < parse_version("2.4.11"):
+                print("Migrating config from < 2.4.11...")
+                if 'api_proxy' not in data:
+                    data["api_proxy"] =""
+                updated = True
+
+            # < 2.4.13
+            if old < parse_version("2.4.13"):
+                print("Migrating config from < 2.4.13...")
+                data["interpreter.auto_clear"] = False
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.code_execute' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.code_execute']
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.code_execute_all' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.code_execute_all']
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.code_execute_file' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.code_execute_file']
+                updated = True
+
+            # < 2.4.14
+            if old < parse_version("2.4.14"):
+                print("Migrating config from < 2.4.14...")
+                if 'prompt.agent.llama.max_eval' not in data:
+                    data["prompt.agent.llama.max_eval"] = self.window.core.config.get_base('prompt.agent.llama.max_eval')
+                if 'prompt.agent.llama.append_eval' not in data:
+                    data["prompt.agent.llama.append_eval"] = self.window.core.config.get_base('prompt.agent.llama.append_eval')
+                if 'agent.llama.loop.enabled' not in data:
+                    data["agent.llama.loop.enabled"] = self.window.core.config.get_base('agent.llama.loop.enabled')
+                if 'agent.llama.loop.score' not in data:
+                    data["agent.llama.loop.score"] = self.window.core.config.get_base('agent.llama.loop.score')
+                updated = True
+
+            # < 2.4.15
+            if old < parse_version("2.4.15"):
+                print("Migrating config from < 2.4.15...")
+                data["interpreter.auto_clear"] = False
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.ipython_execute_new' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.ipython_execute_new']
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.ipython_execute' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.ipython_execute']
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.sys_exec' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.sys_exec']
+                if 'cmd_mouse_control' in data['plugins']:
+                    del data['plugins']['cmd_mouse_control']
+                updated = True
+
+            # < 2.4.16
+            if old < parse_version("2.4.16"):
+                print("Migrating config from < 2.4.16...")
+                data["interpreter.auto_clear"] = False
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.ipython_execute_new' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.ipython_execute_new']
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.ipython_execute' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.ipython_execute']
+                updated = True
+
+            # < 2.4.19
+            if old < parse_version("2.4.19"):
+                print("Migrating config from < 2.4.19...")
+                if 'layout.animation.disable' not in data:
+                    data["layout.animation.disable"] = self.window.core.config.get_base('layout.animation.disable')
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'cmd.ipython_execute' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['cmd.ipython_execute']
+                updated = True
+
+            # < 2.4.21
+            if old < parse_version("2.4.21"):
+                print("Migrating config from < 2.4.21...")
+                if 'ctx.attachment.mode' not in data:
+                    data["ctx.attachment.mode"] = self.window.core.config.get_base('ctx.attachment.mode')
+                if 'ctx.attachment.summary.model' not in data:
+                    data["ctx.attachment.summary.model"] = self.window.core.config.get_base('ctx.attachment.summary.model')
+                if 'ctx.attachment.verbose' not in data:
+                    data["ctx.attachment.verbose"] = self.window.core.config.get_base('ctx.attachment.verbose')
+                updated = True
+
+            # < 2.4.22
+            if old < parse_version("2.4.22"):
+                print("Migrating config from < 2.4.22...")
+                data["ctx.attachment.mode"] = self.window.core.config.get_base('ctx.attachment.mode')
+                if 'ctx.attachment.img' not in data:
+                    data["ctx.attachment.img"] = self.window.core.config.get_base(
+                        'ctx.attachment.img')
+                updated = True
+
+            # < 2.4.29
+            if old < parse_version("2.4.29"):
+                print("Migrating config from < 2.4.29...")
+                if 'cmd_code_interpreter' in data['plugins'] \
+                        and 'ipython_dockerfile' in data['plugins']['cmd_code_interpreter']:
+                    # remove
+                    del data['plugins']['cmd_code_interpreter']['ipython_dockerfile']
+                updated = True
+
+            # < 2.4.31
+            if old < parse_version("2.4.31"):
+                print("Migrating config from < 2.4.31...")
+                if 'attachments_auto_index' not in data:
+                    data["attachments_auto_index"] = self.window.core.config.get_base(
+                        'attachments_auto_index')
+                updated = True
+
+            # < 2.4.34
+            if old < parse_version("2.4.34"):
+                print("Migrating config from < 2.4.34...")
+                if 'ctx.attachment.query.model' not in data:
+                    data["ctx.attachment.query.model"] = self.window.core.config.get_base(
+                        'ctx.attachment.query.model')
+                updated = True
+
+            # < 2.4.35
+            if old < parse_version("2.4.35"):
+                print("Migrating config from < 2.4.35...")
+                data["ctx.edit_icons"] = True
+                updated = True
+
+            # < 2.4.37
+            if old < parse_version("2.4.37"):
+                print("Migrating config from < 2.4.37...")
+                if 'ctx.attachment.rag.history' not in data:
+                    data["ctx.attachment.rag.history"] = self.window.core.config.get_base(
+                        'ctx.attachment.rag.history')
+                if 'ctx.attachment.rag.history.max_items' not in data:
+                    data["ctx.attachment.rag.history.max_items"] = self.window.core.config.get_base(
+                        'ctx.attachment.rag.history.max_items')
+                updated = True
+
+            # < 2.4.38
+            if old < parse_version("2.4.38"):
+                print("Migrating config from < 2.4.38...")
+                if 'theme.style' not in data:
+                    data["theme.style"] = "blocks"
+                if 'audio.input.device' not in data:
+                    data["audio.input.device"] = "0"
+                if 'audio.input.channels' not in data:
+                    data["audio.input.channels"] = 1
+                if 'audio.input.rate' not in data:
+                    data["audio.input.rate"] = 44100
+                self.window.core.updater.patch_css('style.light.css', True)  # force update
+                updated = True
+
+            # < 2.4.39
+            if old < parse_version("2.4.39"):
+                print("Migrating config from < 2.4.39...")
+                if 'layout.split' not in data:
+                    data["layout.split"] = False
+                updated = True
+
+            # < 2.4.40
+            if old < parse_version("2.4.40"):
+                print("Migrating config from < 2.4.40...")
+                if 'cmd_web' in data['plugins'] \
+                        and 'max_result_length' in data['plugins']['cmd_web']:
+                    del data['plugins']['cmd_web']['max_result_length']
+                if 'cmd_web' in data['plugins'] \
+                        and 'cmd.web_search' in data['plugins']['cmd_web']:
+                    del data['plugins']['cmd_web']['cmd.web_search']
+                if 'cmd_web' in data['plugins'] \
+                        and 'cmd.web_url_open' in data['plugins']['cmd_web']:
+                    del data['plugins']['cmd_web']['cmd.web_url_open']
+                if 'cmd_web' in data['plugins'] \
+                        and 'cmd.web_url_raw' in data['plugins']['cmd_web']:
+                    del data['plugins']['cmd_web']['cmd.web_url_raw']
+                self.window.core.updater.patch_css('web-blocks.css', True)  # force update
+                self.window.core.updater.patch_css('web-blocks.light.css', True)  # force update
+                self.window.core.updater.patch_css('web-chatgpt.css', True)  # force update
+                self.window.core.updater.patch_css('web-chatgpt_wide.css', True)  # force update
+                updated = True
+
+            # < 2.4.44
+            if old < parse_version("2.4.44"):
+                print("Migrating config from < 2.4.44...")
+                data["ctx.records.folders.top"] = True
+                updated = True
+
+            # < 2.4.45
+            if old < parse_version("2.4.45"):
+                print("Migrating config from < 2.4.45...")
+                self.window.core.updater.patch_css('style.css', True)  # force update
+                updated = True
+
+            # < 2.4.46
+            if old < parse_version("2.4.46"):
+                print("Migrating config from < 2.4.46...")
+                if 'api_azure_version' not in data:
+                    data["api_azure_version"] = "2023-07-01-preview"
+                if 'api_azure_endpoint' not in data:
+                    data["api_azure_endpoint"] = "https://<your-resource-name>.openai.azure.com/"
+                if 'api_key_google' not in data:
+                    data["api_key_google"] = ""
+                if 'api_key_anthropic' not in data:
+                    data["api_key_anthropic"] = ""
+                if 'api_key_hugging_face' not in data:
+                    data["api_key_hugging_face"] = ""
+                updated = True
+
+            # < 2.4.51
+            if old < parse_version("2.4.51"):
+                print("Migrating config from < 2.4.51...")
+                if 'audio.input.stop_interval' not in data:
+                    data["audio.input.stop_interval"] = 10
+                if 'audio.input.continuous' not in data:
+                    data["audio.input.continuous"] = False
+
+            # < 2.4.55
+            if old < parse_version("2.4.55"):
+                print("Migrating config from < 2.4.55...")
+                if 'audio.input.timeout' not in data:
+                    data["audio.input.timeout"] = 120
+                if 'audio.input.timeout.continuous' not in data:
+                    data["audio.input.timeout.continuous"] = False
+
+            # < 2.4.56
+            if old < parse_version("2.4.56"):
+                print("Migrating config from < 2.4.56...")
+                remove_modifiers = ["Meta", "Keypad", "GroupSwitch"]
+                if 'access.shortcuts' in data:
+                    for item in data['access.shortcuts']:
+                        if 'key_modifier' in item and item['key_modifier'] == 'Control':
+                            item['key_modifier'] = 'Ctrl'
+                        elif 'key_modifier' in item and item['key_modifier'] in remove_modifiers:
+                            item['key_modifier'] = ''
+                updated = True
+
+            # < 2.5.0
+            if old < parse_version("2.5.0"):
+                print("Migrating config from < 2.5.0...")
+                if 'api_key_deepseek' not in data:
+                    data["api_key_deepseek"] = ""
                 updated = True
 
         # update file
