@@ -6,9 +6,11 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.02.01 00:00:00                  #
+# Updated Date: 2024.12.14 22:00:00                  #
 # ================================================== #
 
+from typing import Optional, Dict
+
 from packaging.version import Version
 
 from pygpt_net.item.preset import PresetItem
@@ -32,16 +34,16 @@ class BaseProvider:
     def create(self, item: PresetItem):
         pass
 
-    def load(self) -> dict:
+    def load(self) -> Optional[Dict[str, PresetItem]]:
         pass
 
-    def load_base(self) -> dict:
+    def load_base(self) -> Optional[Dict[str, PresetItem]]:
         pass
 
     def save(self, id: str, item: PresetItem):
         pass
 
-    def save_all(self, items: dict):
+    def save_all(self, items: Dict[str, PresetItem]):
         pass
 
     def remove(self, id: str):
