@@ -6,12 +6,19 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.08.20 16:00:00                  #
+# Updated Date: 2025.01.16 01:00:00                  #
 # ================================================== #
 
-from llama_index.llms.gemini import Gemini
+from typing import Optional, List, Dict
 
+from llama_index.llms.gemini import Gemini
 from llama_index.core.llms.llm import BaseLLM as LlamaBaseLLM
+from llama_index.core.base.embeddings.base import BaseEmbedding
+from llama_index.embeddings.gemini import GeminiEmbedding
+
+from pygpt_net.core.types import (
+    MODE_LLAMA_INDEX,
+)
 from pygpt_net.provider.llms.base import BaseLLM
 from pygpt_net.item.model import ModelItem
 
@@ -27,9 +34,14 @@ class GoogleLLM(BaseLLM):
             - api_key: API key for Google API
         """
         self.id = "google"
-        self.type = ["llama_index"]
+        self.type = [MODE_LLAMA_INDEX, "embeddings"]
 
-    def llama(self, window, model: ModelItem, stream: bool = False) -> LlamaBaseLLM:
+    def llama(
+            self,
+            window,
+            model: ModelItem,
+            stream: bool = False
+    ) -> LlamaBaseLLM:
         """
         Return LLM provider instance for llama
 
@@ -40,3 +52,22 @@ class GoogleLLM(BaseLLM):
         """
         args = self.parse_args(model.llama_index)
         return Gemini(**args)
+
+    def get_embeddings_model(
+            self,
+            window,
+            config: Optional[List[Dict]] = None
+    ) -> BaseEmbedding:
+        """
+        Return provider instance for embeddings
+
+        :param window: window instance
+        :param config: config keyword arguments list
+        :return: Embedding provider instance
+        """
+        args = {}
+        if config is not None:
+            args = self.parse_args({
+                "args": config,
+            })
+        return GeminiEmbedding(**args)
