@@ -6,15 +6,23 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.08.20 16:00:00                  #
+# Updated Date: 2025.01.31 19:00:00                  #
 # ================================================== #
 
+import os
+from typing import Optional, List, Dict
+
 from langchain_community.chat_models import ChatOllama
 
 from llama_index.llms.ollama import Ollama
 from llama_index.core.llms.llm import BaseLLM as LlamaBaseLLM
 from llama_index.core.base.embeddings.base import BaseEmbedding
 from llama_index.embeddings.ollama import OllamaEmbedding
+
+from pygpt_net.core.types import (
+    MODE_LANGCHAIN,
+    MODE_LLAMA_INDEX,
+)
 from pygpt_net.provider.llms.base import BaseLLM
 from pygpt_net.item.model import ModelItem
 import nest_asyncio
@@ -24,9 +32,14 @@ class OllamaLLM(BaseLLM):
     def __init__(self, *args, **kwargs):
         super(OllamaLLM, self).__init__(*args, **kwargs)
         self.id = "ollama"
-        self.type = ["langchain", "llama_index", "embeddings"]
+        self.type = [MODE_LANGCHAIN, MODE_LLAMA_INDEX, "embeddings"]
 
-    def completion(self, window, model: ModelItem, stream: bool = False):
+    def completion(
+            self,
+            window,
+            model: ModelItem,
+            stream: bool = False
+    ):
         """
         Return LLM provider instance for completion
 
@@ -37,7 +50,12 @@ class OllamaLLM(BaseLLM):
         """
         return None
 
-    def chat(self, window, model: ModelItem, stream: bool = False):
+    def chat(
+            self,
+            window,
+            model: ModelItem,
+            stream: bool = False
+    ):
         """
         Return LLM provider instance for chat
 
@@ -49,7 +67,12 @@ class OllamaLLM(BaseLLM):
         args = self.parse_args(model.langchain)
         return ChatOllama(**args)
 
-    def llama(self, window, model: ModelItem, stream: bool = False) -> LlamaBaseLLM:
+    def llama(
+            self,
+            window,
+            model: ModelItem,
+            stream: bool = False
+    ) -> LlamaBaseLLM:
         """
         Return LLM provider instance for llama
 
@@ -60,9 +83,15 @@ class OllamaLLM(BaseLLM):
         """
         nest_asyncio.apply()
         args = self.parse_args(model.llama_index)
+        if "request_timeout" not in args:
+            args["request_timeout"] = 120
         return Ollama(**args)
 
-    def get_embeddings_model(self, window, config: list = None) -> BaseEmbedding:
+    def get_embeddings_model(
+            self,
+            window,
+            config: Optional[List[Dict]] = None
+    ) -> BaseEmbedding:
         """
         Return provider instance for embeddings
 
@@ -76,3 +105,22 @@ class OllamaLLM(BaseLLM):
                 "args": config,
             })
         return OllamaEmbedding(**args)
+
+    def init_embeddings(
+            self,
+            window,
+            env: Optional[List[Dict]] = None
+    ):
+        """
+        Initialize embeddings provider
+
+        :param window: window instance
+        :param env: ENV configuration list
+        """
+        super(OllamaLLM, self).init_embeddings(window, env)
+
+        # === FIX FOR LOCAL EMBEDDINGS ===
+        # if there is no OpenAI api key then set fake key to prevent empty key Llama-index error
+        if ('OPENAI_API_KEY' not in os.environ
+                and (window.core.config.get('api_key') is None or window.core.config.get('api_key') == "")):
+            os.environ['OPENAI_API_KEY'] = "_"
