@@ -6,10 +6,10 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.02.26 22:00:00                  #
+# Updated Date: 2025.01.18 05:00:00                  #
 # ================================================== #
 
-from pygpt_net.ui.widget.dialog.snap import SnapDialogCamera, SnapDialogAudioInput
+from pygpt_net.ui.widget.dialog.snap import SnapDialogCamera, SnapDialogAudioInput, SnapDialogAudioOutput
 
 
 class Snap:
@@ -25,3 +25,4 @@ class Snap:
         """Setup snap dialog"""
         self.window.ui.dialog['snap_camera'] = SnapDialogCamera(self.window)
         self.window.ui.dialog['snap_audio_input'] = SnapDialogAudioInput(self.window)
+        self.window.ui.dialog['snap_audio_output'] = SnapDialogAudioOutput(self.window)
