@@ -6,13 +6,17 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.08 17:00:00                  #
+# Updated Date: 2024.11.21 20:00:00                  #
 # ================================================== #
 
 from PySide6 import QtCore
 from PySide6.QtGui import QStandardItemModel, Qt, QIcon
-from PySide6.QtWidgets import QVBoxLayout, QHBoxLayout, QPushButton, QSplitter, QWidget
+from PySide6.QtWidgets import QVBoxLayout, QHBoxLayout, QPushButton, QSplitter, QWidget, QSizePolicy
 
+from pygpt_net.core.types import (
+    MODE_AGENT,
+    MODE_EXPERT,
+)
 from pygpt_net.ui.widget.element.labels import HelpLabel, TitleLabel
 from pygpt_net.ui.widget.lists.preset import PresetList
 from pygpt_net.ui.layout.toolbox.footer import Footer
@@ -43,7 +47,8 @@ class Presets:
 
         self.window.ui.nodes['presets.widget'] = QWidget()
         self.window.ui.nodes['presets.widget'].setLayout(presets)
-        self.window.ui.nodes['presets.widget'].setMinimumHeight(150)
+        self.window.ui.nodes['presets.widget'].setMinimumHeight(180)
+        self.window.ui.nodes['presets.widget'].setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
 
         return self.window.ui.nodes['presets.widget']
 
@@ -70,20 +75,24 @@ class Presets:
         header.addWidget(self.window.ui.nodes['preset.agents.label'])
         header.addWidget(self.window.ui.nodes['preset.experts.label'])
         header.addWidget(self.window.ui.nodes['preset.presets.new'], alignment=Qt.AlignRight)
-        header.setContentsMargins(0, 0, 0, 0)
+        header.setContentsMargins(5, 0, 0, 0)
 
         header_widget = QWidget()
         header_widget.setLayout(header)
 
         self.window.ui.nodes[self.id] = PresetList(self.window, self.id)
         self.window.ui.nodes[self.id].selection_locked = self.window.controller.presets.preset_change_locked
+        self.window.ui.nodes[self.id].setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
 
         self.window.ui.nodes['tip.toolbox.presets'] = HelpLabel(trans('tip.toolbox.presets'), self.window)
+        self.window.ui.nodes['tip.toolbox.presets'].setAlignment(Qt.AlignCenter)
 
         layout = QVBoxLayout()
+        layout.addStretch()
         layout.addWidget(header_widget)
-        layout.addWidget(self.window.ui.nodes[self.id])
+        layout.addWidget(self.window.ui.nodes[self.id], 1)
         layout.addWidget(self.window.ui.nodes['tip.toolbox.presets'])
+        layout.setContentsMargins(2, 5, 5, 5)
 
         self.window.ui.models[self.id] = self.create_model(self.window)
         self.window.ui.nodes[self.id].setModel(self.window.ui.models[self.id])
@@ -115,9 +124,9 @@ class Presets:
             name = data[n].name
 
             # show disabled in expert mode
-            if mode == "expert" and not n.startswith("current.") and data[n].enabled:
+            if mode == MODE_EXPERT and not n.startswith("current.") and data[n].enabled:
                 name = "[x] " + name
-            elif mode =="agent":
+            elif mode == MODE_AGENT:
                 num_experts = self.window.core.experts.count_experts(n)
                 if num_experts > 0:
                     name = name + " (" + str(num_experts) + " experts)"
