@@ -6,15 +6,19 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.05 23:00:00                  #
+# Updated Date: 2025.01.19 03:00:00                  #
 # ================================================== #
 
+import copy
 import os
 
+from PySide6 import QtWidgets
 from PySide6.QtCore import QTimer, Signal, Slot, QThreadPool, QEvent, Qt, QLoggingCategory
+from PySide6.QtGui import QShortcut, QKeySequence
 from PySide6.QtWidgets import QApplication, QMainWindow
 from qt_material import QtStyleTools
 
+from pygpt_net.core.events import BaseEvent, KernelEvent, ControlEvent
 from pygpt_net.container import Container
 from pygpt_net.controller import Controller
 from pygpt_net.tools import Tools
@@ -84,6 +88,9 @@ class MainWindow(QMainWindow, QtStyleTools):
         self.ui = UI(self)
         self.ui.init()
 
+        # global shortcuts
+        self.shortcuts = []
+
         # setup signals
         self.statusChanged.connect(self.update_status)
         self.stateChanged.connect(self.update_state)
@@ -119,7 +126,7 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_plugin(self, plugin):
         """
-        Add plugin to app
+        Add a plugin to the app
 
         :param plugin: plugin instance
         """
@@ -127,7 +134,7 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_llm(self, llm):
         """
-        Add Langchain LLM wrapper to app
+        Add a Langchain LLM wrapper to the app
 
         :param llm: LLM wrapper instance
         """
@@ -135,7 +142,7 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_vector_store(self, store):
         """
-        Add vector store provider to app
+        Add a vector store provider to the app
 
         :param store: Vector store provider instance
         """
@@ -143,15 +150,23 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_loader(self, loader):
         """
-        Add data loader to app
+        Add a data loader to the app
 
         :param loader: data loader instance
         """
         self.core.idx.indexing.register_loader(loader)
 
+    def add_agent(self, agent):
+        """
+        Add an agent to the app
+
+        :param agent: agent instance
+        """
+        self.core.agents.provider.register(agent.id, agent)
+
     def add_audio_input(self, provider):
         """
-        Add audio input provider to app
+        Add an audio input provider to the app
 
         :param provider: audio input provider instance
         """
@@ -159,7 +174,7 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_audio_output(self, provider):
         """
-        Add audio output provider to app
+        Add an audio output provider to the app
 
         :param provider: audio output provider instance
         """
@@ -167,7 +182,7 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_web(self, provider):
         """
-        Add web provider to app
+        Add a web provider to the app
 
         :param provider: web provider instance
         """
@@ -175,7 +190,7 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def add_tool(self, tool):
         """
-        Add tool to app
+        Add a tool to the app
 
         :param tool: tool instance
         """
@@ -183,8 +198,8 @@ class MainWindow(QMainWindow, QtStyleTools):
 
     def setup(self):
         """Setup app"""
-        self.controller.setup()
         self.tools.setup()
+        self.controller.setup()
         self.controller.plugins.setup()
         self.controller.post_setup()
 
@@ -217,13 +232,13 @@ class MainWindow(QMainWindow, QtStyleTools):
         self.tools.on_post_update()
 
     @Slot(str)
-    def update_status(self, text: str):
+    def update_status(self, message: str = ""):
         """
-        Update status text
+        Update global status
 
-        :param text: status text
+        :param message: status message
         """
-        self.ui.status(text)
+        self.dispatch(KernelEvent(KernelEvent.STATUS, {"status": str(message)}))
 
     @Slot(str)
     def update_state(self, state: str):
@@ -235,6 +250,16 @@ class MainWindow(QMainWindow, QtStyleTools):
         self.state = state
         self.ui.tray.set_icon(state)
 
+    @Slot(object)
+    def dispatch(self, event: BaseEvent, all: bool = False):
+        """
+        Dispatch App event
+
+        :param event
+        :param all: True to dispatch to all plugins
+        """
+        self.core.dispatcher.dispatch(event, all=all)
+
     def closeEvent(self, event):
         """
         Handle close event
@@ -248,9 +273,8 @@ class MainWindow(QMainWindow, QtStyleTools):
 
         self.is_closing = True
         print("Closing...")
-        print("Sending terminate signal to all plugins...")
-        self.controller.chat.common.stop(exit=True)
-        self.controller.plugins.destroy()
+        print("Sending terminate signal to all...")
+        self.controller.kernel.terminate()
         print("Saving ctx groups...")
         self.controller.ctx.save_all()
         print("Saving tabs...")
@@ -320,3 +344,57 @@ class MainWindow(QMainWindow, QtStyleTools):
             self.showNormal()
         self.activateWindow()
         self.ui.tray_menu['restore'].setVisible(False)
+
+    def setup_global_shortcuts(self):
+        """Setup global shortcuts"""
+        if not hasattr(self, 'core') or not hasattr(self.core, 'config'):
+            return
+
+        # unregister existing shortcuts
+        if hasattr(self, 'shortcuts'):
+            for shortcut in self.shortcuts:
+                # disconnect signals
+                shortcut.activated.disconnect()
+                # disable the shortcut to prevent it from handling events
+                shortcut.setEnabled(False)
+                # remove the parent to break the QObject tree
+                shortcut.setParent(None)
+                # schedule the shortcut for deletion
+                shortcut.deleteLater()
+            # clear the list of shortcuts
+            self.shortcuts.clear()
+            # process events to delete shortcuts immediately
+            QtWidgets.QApplication.processEvents()
+        else:
+            self.shortcuts = []
+
+        # Handle the Escape key
+        escape_shortcut = QShortcut(QKeySequence(Qt.Key_Escape), self)
+        escape_shortcut.setContext(Qt.ApplicationShortcut)
+        escape_shortcut.activated.connect(self.controller.access.on_escape)
+        self.shortcuts.append(escape_shortcut)
+
+        config = copy.deepcopy(self.core.config.get("access.shortcuts"))
+        for shortcut_conf in config:
+            key = shortcut_conf.get('key', '')
+            key_modifier = shortcut_conf.get('key_modifier', '')
+            action_name = shortcut_conf.get('action')
+
+            if not key or not action_name:
+                continue
+
+            key_sequence_parts = []
+            if key_modifier and key_modifier != '---':
+                if key_modifier == "Control":
+                    key_modifier = "Ctrl"
+                key_sequence_parts.append(key_modifier)
+            key_sequence_parts.append(key)
+            key_sequence_str = '+'.join(key_sequence_parts)
+            key_sequence = QKeySequence(key_sequence_str)
+
+            shortcut = QShortcut(key_sequence, self)
+            shortcut.setContext(Qt.ApplicationShortcut)
+            shortcut.activated.connect(
+                lambda checked=False, action=action_name: self.dispatch(ControlEvent(action))
+            )
+            self.shortcuts.append(shortcut)
