@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.08.22 17:00:00                  #
+# Updated Date: 2024.12.16 01:00:00                  #
 # ================================================== #
 
 import datetime
@@ -14,6 +14,7 @@ import datetime
 from PySide6 import QtWidgets, QtCore, QtGui
 from PySide6.QtGui import QAction, QIcon, QColor, QPixmap, QStandardItem
 from PySide6.QtWidgets import QMenu
+from overrides import overrides
 
 from pygpt_net.ui.widget.lists.base import BaseList
 from pygpt_net.utils import trans
@@ -31,13 +32,13 @@ class ContextList(BaseList):
         super(ContextList, self).__init__(window)
         self.window = window
         self.id = id
-        self.clicked.connect(self.click)
         self.expanded_items = set()
         self.setItemDelegate(ImportantItemDelegate())
 
+
     def click(self, index):
         """
-        Click event
+        Click event (override, connected in BaseList class)
 
         :param index: index
         """
@@ -49,8 +50,10 @@ class ContextList(BaseList):
                 self.window.controller.ctx.set_group(item.id)
                 if self.window.ui.nodes['ctx.list'].isExpanded(index):
                     self.expanded_items.discard(item.id)
+                    self.window.ui.nodes['ctx.list'].collapse(index)
                 else:
                     self.expanded_items.add(item.id)
+                    self.window.ui.nodes['ctx.list'].expand(index)
             else:
                 self.window.controller.ctx.select_by_id(item.id)
         else:
@@ -76,7 +79,7 @@ class ContextList(BaseList):
 
         :param index: index
         """
-        pass
+        print("dblclick")
 
     def mousePressEvent(self, event):
         """
@@ -175,6 +178,11 @@ class ContextList(BaseList):
                     lambda checked=False, ctx_id=ctx_id: self.action_copy_id(ctx_id)
                 )
 
+                actions['reset'] = QAction(QIcon(":/icons/close.svg"), trans('action.ctx_reset'), self)
+                actions['reset'].triggered.connect(
+                    lambda checked=False, ctx_id=ctx_id: self.action_reset(ctx_id)
+                )
+
                 menu = QMenu(self)
                 menu.addAction(actions['rename'])
                 menu.addAction(actions['duplicate'])
@@ -295,6 +303,8 @@ class ContextList(BaseList):
                     action.setEnabled(False)  # disable action, only for info
                     menu.addAction(action)
 
+                menu.addAction(actions['reset'])
+
                 if idx >= 0:
                     self.window.controller.ctx.select_by_id(ctx_id)
                     menu.exec_(event.globalPos())
@@ -373,6 +383,14 @@ class ContextList(BaseList):
         """
         self.window.controller.ctx.common.copy_id(id)
 
+    def action_reset(self, id):
+        """
+        Reset action handler
+
+        :param id: context id
+        """
+        self.window.controller.ctx.common.reset(id)
+
 
 class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
     """
@@ -387,11 +405,40 @@ class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
         super(ImportantItemDelegate, self).paint(painter, option, index)
 
         # pin (>= 10)
-        if index.data(QtCore.Qt.ItemDataRole.UserRole) and index.data(QtCore.Qt.ItemDataRole.UserRole) > 0:
-            label = index.data(QtCore.Qt.ItemDataRole.UserRole)
+        if index.data(QtCore.Qt.ItemDataRole.UserRole):
+            data = index.data(QtCore.Qt.ItemDataRole.UserRole)
+            label = 0
+            is_important = False
+            is_attachment = False
+            is_group = False
+            in_group = False
+
+            if "label" in data:
+                label = data["label"]
+            if "is_important" in data and data["is_important"]:
+                is_important = True
+            if "is_attachment" in data and data["is_attachment"]:
+                is_attachment = True
+            if "is_group" in data and data["is_group"]:
+                is_group = True
+            if "in_group" in data and data["in_group"]:
+                in_group = True
+
             painter.save()
 
-            if label >= 10:
+            if is_attachment:
+                icon = QtGui.QIcon(":/icons/attachment.svg")
+                icon_size = option.decorationSize or QtCore.QSize(16, 16)
+                icon_pos = option.rect.right() - icon_size.width()
+                icon_rect = QtCore.QRect(
+                    icon_pos,
+                    option.rect.top() + (option.rect.height() - icon_size.height()) / 2,
+                    icon_size.width(),
+                    icon_size.height()
+                )
+                icon.paint(painter, icon_rect, QtCore.Qt.AlignCenter)
+
+            if is_important:
                 color = self.get_color_for_status(3)
                 square_size = 3
                 square_margin = 0
@@ -411,8 +458,6 @@ class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
                 )
                 painter.drawRect(square_rect)
 
-                label = label - 10  # remove pin status
-
             # label (0-9)
             if label > 0:
                 color = self.get_color_for_status(label)
@@ -459,6 +504,7 @@ class GroupItem(QStandardItem):
         self.name = name
         self.isFolder = True
         self.isPinned = False
+        self.hasAttachments = False
         self.dt = None
 
 class Item(QStandardItem):
@@ -471,11 +517,16 @@ class Item(QStandardItem):
         self.dt = None
 
 class SectionItem(QStandardItem):
-    def __init__(self, title):
+    def __init__(self, title, group: bool = False):
         super().__init__(title)
         self.title = title
+        self.group = group
         self.setSelectable(False)
         self.setEnabled(False)
-        self.setTextAlignment(QtCore.Qt.AlignRight)
+        if self.group:
+            self.setTextAlignment(QtCore.Qt.AlignLeft)
+        else:
+            self.setTextAlignment(QtCore.Qt.AlignRight)
         font = self.font()
         font.setBold(True)
+        self.setFont(font)
