@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.08.29 04:00:00                  #
+# Updated Date: 2025.02.01 02:00:00                  #
 # ================================================== #
 
 import json
@@ -93,12 +93,17 @@ def get_app_meta() -> dict:
         'docs': get_init_value("__documentation__"),
         'pypi': get_init_value("__pypi__"),
         'snap': get_init_value("__snap__"),
+        'ms_store': get_init_value("__ms_store__"),
         'donate': get_init_value("__donate__"),
         'discord': get_init_value("__discord__"),
         'version': get_init_value("__version__"),
         'build': get_init_value("__build__"),
         'author': get_init_value("__author__"),
-        'email': get_init_value("__email__")
+        'email': get_init_value("__email__"),
+        'donate_coffee': get_init_value("__donate_coffee__"),
+        'donate_paypal': get_init_value("__donate_paypal__"),
+        'donate_github': get_init_value("__donate_github__"),
+        'report': get_init_value("__report__"),
     }
 
 
@@ -137,12 +142,21 @@ def parse_args(data: list) -> dict:
                 except Exception:
                     args[key] = False
         elif type == 'dict':
-            try:
-                args[key] = json.loads(value)
-            except json.JSONDecodeError:
-                args[key] = {}
+            if isinstance(value, dict):
+                args[key] = value
+            else:
+                try:
+                    args[key] = json.loads(value)
+                except:
+                    args[key] = {}
         elif type == 'list':
-            args[key] = [x.strip() for x in value.split(',')]
+            if isinstance(value, list):
+                args[key] = value
+            else:
+                try:
+                    args[key] = [x.strip() for x in value.split(',')]
+                except:
+                    args[key] = []
         elif type == 'None':
             args[key] = None
         else:
@@ -221,3 +235,13 @@ def get_tz_offset() -> int:
     now = datetime.now()
     now_timestamp = int(now.timestamp())
     return now_timestamp - utc_timestamp
+
+def natsort(l: list) -> list:
+    """
+    Sort the given list in natural order
+
+    :param l: list to sort
+    """
+    convert = lambda text: int(text) if text.isdigit() else text.lower()
+    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
+    return sorted(l, key=alphanum_key)
\ No newline at end of file
